SystemOrganization addCategory: #'XML-Tests-Parser'!
SystemOrganization addCategory: 'XML-Tests-Parser-DOM'!

TestCase subclass: #XMLEncodingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLEncodingTest methodsFor: 'private' stamp: 'monty 1/12/2014 23:15'!
convertableStreamOn: aString
	^ (MultiByteBinaryOrTextStream on: aString from: 1 to: aString size)
		text;
		yourself! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 1/21/2014 02:42'!
testDecodingCharacters
	| xml document  |

	xml := '<?xml version="1.0" encoding="UTF-8"?>
			<test-data>&#{1};</test-data>'.

	document := XMLDOMParser parse: (xml format: (Array with: '8230')).
	self assert: document root contentString first charCode = 8230.

	self
		should: [XMLDOMParser parse: (xml format: (Array with: ''))]
		raise: XMLWellFormednessException.
	self
		should: [XMLDOMParser parse: (xml format: (Array with: '0'))]
		raise: XMLWellFormednessException.
	self
		should: [XMLDOMParser parse: (xml format: (Array with: '-1'))]
		raise: XMLWellFormednessException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 1/12/2014 23:15'!
testEncodingDeclaration
	| xml utf8Xml latin1Xml |

	xml :=
		'<?xml version="1.0" encoding="{1}"?><root>',
		(String with: (Character value: 153) with: (Character value: 79)),
		'</root>'.
	utf8Xml := xml format: (Array with: 'UTF-8').
	latin1Xml := xml format: (Array with: 'ISO-8859-1').
	self
		should: [SAXHandler parse: (self convertableStreamOn: utf8Xml)]
		raise: Error.
	self
		shouldnt: [SAXHandler parse: (self convertableStreamOn: latin1Xml)]
		raise: Error.! !

TestCase subclass: #XMLExternalEntityTest
	instanceVariableNames: 'isSkipping'
	classVariableNames: 'IsSkipping'
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

XMLExternalEntityTest subclass: #XMLExternalEntityLimitsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 1/26/2014 04:29'!
testDefaultLimits
	| resolver |

	resolver := (SAXHandler on: '') externalEntityResolver.
	self
		assert: resolver maxExternalEntities > 0;
		assert: resolver maxExternalEntitySize = DTDExternalEntityLimits defaultMaxExternalEntitySize;
		assert: (resolver resolvesLocalFromNonLocal isKindOf: Boolean).! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 2/11/2014 01:02'!
testExternalEntitiesLimit
	| parser xml entityLimit |

	self isSkipping
		ifTrue: [self skip].

	entityLimit := 2.
	xml := self doctypeWithDeclarationsForEntityURIs: (self externalEntityURLs first: entityLimit).
	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntities: entityLimit.
	self
		shouldnt: [parser parseDocument]
		raise: XMLLimitException.

	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntities: entityLimit - 1.
	self
		should: [parser parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:56'!
testExternalEntitySizeLimit
	| parser url xml size |

	self isSkipping
		ifTrue: [self skip].

	url := self externalEntityURLs first asXMLURI.
	xml := self doctypeWithDeclarationsForEntityURIs: (Array with: url asString).
	size := url get contents size.
	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntitySize: size.
	self
		shouldnt: [parser parseDocument]
		raise: XMLLimitException.

	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntitySize: size - 1.
	self
		should: [parser parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:56'!
testExternalSubsetReadLimit
	| xml externalSubset fileName parser |

	self isSkipping
		ifTrue: [self skip].

	fileName := 'externalSubset.dtd'.
	externalSubset := '<?xml encoding="UTF-8"?><!!ELEMENT test EMPTY>'.
	self fileNamed: fileName write: externalSubset.
	xml := '<!!DOCTYPE test SYSTEM "', fileName, '"><test/>'.
	self
		shouldnt: [
			(parser := SAXHandler on: xml)
				externalSubsetReadLimit: externalSubset size;
				resolvesExternalEntities: true;
				parseDocument]
		raise: XMLLimitException.
	self
		should: [
			(parser := SAXHandler on: xml)
				externalSubsetReadLimit: externalSubset size - 1;
				resolvesExternalEntities: true;
				parseDocument]
		raise: XMLLimitException.
	self deleteFileNamed: fileName.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 9/29/2014 12:12'!
testOnURLUpToLimit
	| parser size |

	self isSkipping
		ifTrue: [self skip].

	size := (self feedURL asXMLURI get) contents size.
	self
		shouldnt: [
			parser :=
				SAXHandler
					onURL: self feedURL
					upToLimit: size]
		raise: XMLLimitException.
	self assert: parser documentReadLimit = size.

	self
		should: [
			(SAXHandler
				onURL: self feedURL
				upToLimit: size - 1)
				parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 1/24/2014 07:03'!
testRemoveLimits
	|  parser resolver entityLimit entitySizeLimit |

	parser := SAXHandler new.
	entityLimit := 10.
	entitySizeLimit := 100.
	(resolver := parser externalEntityResolver)
		maxExternalEntities: entityLimit;
		maxExternalEntitySize: entitySizeLimit;
		resolvesLocalFromNonLocal: true.
	self
		assert: resolver maxExternalEntities = entityLimit;
		assert: resolver maxExternalEntitySize = entitySizeLimit;
		assert: resolver resolvesLocalFromNonLocal.

	parser removeLimits.
	self
		assert: resolver maxExternalEntities isNil;
		assert: resolver maxExternalEntitySize isNil;
		assert: resolver resolvesLocalFromNonLocal isNil.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 1/24/2014 17:33'!
testResolvingLocalFromNonLocal
	| parser |

	self
		should: [
			(parser :=
				SAXHandler
					on: '<!!DOCTYPE root SYSTEM "file://local/uri">'
					documentURI: 'http://www.example.com')
				externalEntityResolver resolvesLocalFromNonLocal: false.
			parser
				resolvesExternalEntities: true;
				parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityTest class methodsFor: 'testing' stamp: 'monty 2/11/2014 02:25'!
isSkipping
	"HTTP/file tests are skipped by default"
	^ IsSkipping ifNil: [IsSkipping := true]! !

!XMLExternalEntityTest class methodsFor: 'testing' stamp: 'monty 2/11/2014 00:57'!
isSkipping: aBoolean
	"
	XMLExternalEntityTest isSkipping: false.
	XMLExternalEntityTest isSkipping: true.
	"
	IsSkipping := aBoolean! !

!XMLExternalEntityTest methodsFor: 'files' stamp: 'monty 1/21/2014 23:36'!
deleteFileNamed: aFileName
	(XMLFileHandle path: aFileName) delete! !

!XMLExternalEntityTest methodsFor: 'source' stamp: 'monty 1/21/2014 22:18'!
doctypeWithDeclarationsForEntityURIs: aUrlCollection
	^  '<!!DOCTYPE test [',
		('' join: (aUrlCollection collectWithIndex: [:each :i |
			'<!!ENTITY entity', i printString, ' SYSTEM "', each, '">'])),
		']>'! !

!XMLExternalEntityTest methodsFor: 'urls' stamp: 'monty 1/13/2014 02:04'!
externalEntityURLs
	^  #('http://www.musicxml.org/dtds/note.mod'
		'http://www.musicxml.org/dtds/isolat1.ent'
		'http://www.musicxml.org/dtds/isolat2.ent'
		'http://www.musicxml.org/dtds/layout.mod'
		'http://www.musicxml.org/dtds/identity.mod'
		'http://www.musicxml.org/dtds/attributes.mod'
		'http://www.musicxml.org/dtds/partwise.dtd'
		'http://www.musicxml.org/dtds/barline.mod'
		'http://www.musicxml.org/dtds/direction.mod'
		'http://www.musicxml.org/dtds/score.mod'
		'http://www.musicxml.org/dtds/link.mod'
		'http://www.musicxml.org/dtds/common.mod')! !

!XMLExternalEntityTest methodsFor: 'urls' stamp: 'monty 9/29/2014 12:33'!
feedURL
	^ 'http://pharo.org/news/atom.xml'! !

!XMLExternalEntityTest methodsFor: 'files' stamp: 'monty 1/22/2014 00:22'!
fileNamed: aFileName write: aString
	(XMLFileHandle path: aFileName) writeStream
		nextPutAll: aString;
		close.! !

!XMLExternalEntityTest methodsFor: 'accessing' stamp: 'monty 2/11/2014 00:55'!
isSkipping
	^ self class isSkipping! !

!XMLExternalEntityTest methodsFor: 'source' stamp: 'monty 1/13/2014 01:54'!
sourceXML
^ '<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!!DOCTYPE score-partwise PUBLIC
    "-//Recordare//DTD MusicXML 3.0 Partwise//EN"
    "http://www.musicxml.org/dtds/partwise.dtd">
<score-partwise version="3.0">
  <part-list>
    <score-part id="P1">
      <part-name>Music</part-name>
    </score-part>
  </part-list>
  <part id="P1">
    <measure number="1">
      <attributes>
        <divisions>1</divisions>
        <key>
          <fifths>0</fifths>
        </key>
        <time>
          <beats>4</beats>
          <beat-type>4</beat-type>
        </time>
        <clef>
          <sign>G</sign>
          <line>2</line>
        </clef>
      </attributes>
      <note>
        <pitch>
          <step>C</step>
          <octave>4</octave>
        </pitch>
        <duration>4</duration>
        <type>whole</type>
      </note>
    </measure>
  </part>
</score-partwise>
'.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:55'!
testOnFileNamed
	| parser fileName |

	self isSkipping
		ifTrue: [self skip].

	fileName := 'testOnFileNamed.xml'.
	self fileNamed: fileName write: ''.
	self
		shouldnt: [parser := SAXHandler onFileNamed: fileName]
		raise: XMLFileException.
	self
		assert: parser documentURI = fileName;
		assert: parser documentURI asXMLURI isLocalURI.

	self deleteFileNamed: fileName.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 9/29/2014 12:12'!
testOnURL
	| parser |

	self isSkipping
		ifTrue: [self skip].

	self
		shouldnt: [parser := SAXHandler onURL: self feedURL]
		raise: XMLHTTPException.
	self
		assert: parser documentURI = self feedURL;
		deny: parser documentURI asXMLURI isLocalURI.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 2/11/2014 01:06'!
testParsingExternalEntities
	"This test uses HTTP so is skipped by default."
	| entityCache entityCacheCopy |

	self isSkipping
		ifTrue: [self skip].

	DTDCachingEntityResolver clearEntityCache.
	entityCache := DTDCachingEntityResolver entityCache.
	self externalEntityURLs do: [:each |
		self deny: (entityCache includesKey: each asXMLURI)].

	SAXHandler
		parse: self sourceXML
		usingNamespaces: true
		validation: true
		externalEntities: true.
	self externalEntityURLs do: [:each |
		self assert: (entityCache at: each asXMLURI ifAbsent: ['']) notEmpty].

	"check for re-requesting"
	entityCacheCopy := entityCache copy.
	SAXHandler
		parse: self sourceXML
		usingNamespaces: true
		validation: true
		externalEntities: true.
	self externalEntityURLs do: [:each |
		self assert: (entityCache at: each asXMLURI) == (entityCacheCopy at: each asXMLURI)].! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:56'!
testParsingFile
	"This test creates/deletes a file so is skipped by default"
	| fileName doc |

	self isSkipping
		ifTrue: [self skip].

	fileName := 'testParsingFile.xml'.
	self
		should: [XMLDOMParser parseFileNamed: fileName]
		raise: XMLFileException.
	self
		fileNamed: fileName
		write: self sourceXML.
	self
		shouldnt: [doc := XMLDOMParser parseFileNamed: fileName]
		raise: XMLFileException.
	self assert: (doc root isNamed: 'score-partwise').

	self deleteFileNamed: fileName.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 9/29/2014 12:33'!
testParsingURL
	"This uses HTTP so is skipped by default"
	| doc |

	self isSkipping
		ifTrue: [self skip].

	self
		shouldnt: [doc := XMLDOMParser parseURL: self feedURL]
		raise: XMLHTTPException.
	self assert: (doc root isNamed: 'feed').! !

TestCase subclass: #XMLNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLNodeTest commentStamp: '<historical>' prior: 0!
This is a base class for node tests.!

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 10/30/2010 21:51'!
assertNode: aNode hasName: aQualifiedName namespaceURI: aUri
	| prefix localName |

	prefix := aQualifiedName copyUpTo: $:.
	prefix = aQualifiedName
		ifTrue: [
			prefix := ''.
			localName := aQualifiedName]
		ifFalse: [localName := aQualifiedName copyAfter: $:].
	self
		assert: aNode name = aQualifiedName;
		assert: aNode prefix = prefix;
		assert: aNode localName = localName;
		assert: (aNode isNamed: aQualifiedName);
		assert: (aNode isNamed: localName);
		assert: aNode namespaceURI = aUri;
		assert:
			aNode expandedName =
				(aUri
					ifEmpty: [localName]
					ifNotEmpty: ['{', aUri, '}', localName]);
		assert: (prefix ifEmpty: [aNode hasPrefix not] ifNotEmpty: [aNode hasPrefix])

! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/20/2010 18:20'!
assertNode: aNode hasPrefix: aPrefix uri: aUri
	self
		assert: aNode prefix = aPrefix;
		assert: aNode namespaceURI = aUri! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 14:56'!
assertNode: aNode hasType: aType
	self assertNode: aNode hasTypes: (aType ifNotNil: [Array with: aType] ifNil: [#()])! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 11/24/2010 19:47'!
assertNode: aNode hasTypes: aTypeArray
	self deny:
		((#(isCData isComment isContentNode isDocument isElement isPI isStringNode)
			copyWithoutAll: aTypeArray) anySatisfy: [:each | aNode perform: each]).
	self assert: (aTypeArray allSatisfy: [:each | aNode perform: each]).! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/8/2010 07:21'!
createNodes: aNumber with: aBlock
	^ ((1 to: aNumber)
		collect: [:i | aBlock valueWithPossibleArgument: i])
			asOrderedCollection! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 21:21'!
escapedString
	^ 'test&lt;&gt;&amp;&quot;'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 10/20/2010 18:26'!
intersperse: aNodeCollection with: aPaddingCollection
	| mixedCollection nodeStream paddingStream |

	mixedCollection := XMLOrderedList new.
	nodeStream := aNodeCollection readStream.
	paddingStream := aPaddingCollection readStream.
	[nodeStream atEnd and: [paddingStream atEnd]]
		whileFalse: [
			nodeStream atEnd
				ifFalse: [mixedCollection add: nodeStream next].
			paddingStream atEnd
				ifFalse: [mixedCollection add: paddingStream next]].
	^ mixedCollection.! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 09:10'!
intersperseWithNodes: aNodeCollection
	^ self
		intersperse: aNodeCollection
		with: (self newNodes: aNodeCollection size)! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:10'!
markupString
	^ 'test<>&"'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 11:16'!
newNodes: aNumber
	^ self createNodes: aNumber with: [XMLMockNode new]! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:02'!
newStrings: aNumber
	^ self
		createNodes: aNumber
		with: [:i | XMLString string: 'test', i printString]! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:25'!
should: aBlock enumerate: aCollection
	| enumerated |

	enumerated := OrderedCollection new.
	aBlock value: [:each | enumerated add: each].
	self assert: enumerated = aCollection asOrderedCollection.! !

XMLNodeTest subclass: #XMLNodesWithChildrenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLNodesWithChildrenTest subclass: #XMLDeclarationNodesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'monty 12/31/2013 00:10'!
assertAttributeDecl: anAttributeDeclaration hasElementName: anElementName attributeName: anAttributeName type: aType default: aDefaultPragma value: aDefaultValue
	| printString |

	self
		assert: anAttributeDeclaration element = anElementName;
		assert: anAttributeDeclaration attribute = anAttributeName;
		assert: anAttributeDeclaration type = aType;
		assert: anAttributeDeclaration defaultPragma = aDefaultPragma;
		assert: anAttributeDeclaration defaultValue = aDefaultValue.

	(printString := String new writeStream)
		nextPutAll: '<!!ATTLIST';
		space;
		nextPutAll: anElementName;
		space;
		nextPutAll: anAttributeName;
		space;
		nextPutAll: aType.

	aDefaultPragma ifNotEmpty: [
		printString
			space;
			nextPutAll: (aDefaultPragma copyWithFirst: $#)].

	aDefaultValue ifNotEmpty: [
		printString
			space;
			nextPut: $";
			nextPutAll: aDefaultValue;
			nextPut: $"].

	printString nextPut: $>.
		
	self assert: anAttributeDeclaration printString = printString contents.! !

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'monty 12/21/2013 03:24'!
assertElementDecl: anElementDeclaration hasName: aName contentModel: aContentModel
	self
		assert: anElementDeclaration element = aName;
		assert: anElementDeclaration contentModel = aContentModel;
		assert: anElementDeclaration printString = ('<!!ELEMENT ', aName, ' ', aContentModel, '>')! !

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 20:46'!
assertEntityDecl: anEntityDeclaration hasName: aName replacement: aReplacement publicID: aPublicId systemID: aSystemId ndada: anNdata
	| printString |

	self
		assert: anEntityDeclaration name = aName;
		assert: anEntityDeclaration replacement = aReplacement;
		assert: anEntityDeclaration publicID = aPublicId;
		assert: anEntityDeclaration systemID = aSystemId;
		assert: anEntityDeclaration ndata = anNdata.

	(printString := String new writeStream)
		nextPutAll: '<!!ENTITY ';
		nextPutAll: aName;
		space.
	aReplacement
		ifNotEmpty: [
			printString
				nextPut: $";
				nextPutAll: aReplacement;
				nextPut: $"]
		ifEmpty: [
			self
				printExternalIDFromPublicID: aPublicId
				systemID: aSystemId
				on: printString.
			anNdata ifNotEmpty: [
				printString
					space;
					nextPutAll: 'NDATA';
					space;
					nextPutAll: anNdata]].
	printString nextPut: $>.

	self assert: anEntityDeclaration printString = printString contents.! !

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 20:51'!
assertNotationDecl: aNotationDeclaration hasName: aName publicID: aPublicId systemID: aSystemId
	| printString |

	self
		assert: aNotationDeclaration name = aName;
		assert: aNotationDeclaration publicID = aPublicId;
		assert: aNotationDeclaration systemID = aSystemId.

	(printString := String new writeStream)
		nextPutAll: '<!!NOTATION';
		space;
		nextPutAll: aName;
		space.
	self
		printExternalIDFromPublicID: aPublicId
		systemID: aSystemId
		on: printString.
	printString nextPut: $>.

	self assert: aNotationDeclaration printString = printString contents.! !

!XMLDeclarationNodesTest methodsFor: 'printing' stamp: 'JAAyer 10/30/2010 20:39'!
printExternalIDFromPublicID: aPublicId systemID: aSystemId on: aStream
	aPublicId
		ifNotEmpty: [
			aStream
				nextPutAll: 'PUBLIC';
				space;
				nextPut: $";
				nextPutAll: aPublicId;
				nextPut: $"]
		ifEmpty: [aSystemId ifNotEmpty: [aStream nextPutAll: 'SYSTEM']].
	aSystemId ifNotEmpty: [
		aStream
			space;
			nextPut: $";
			nextPutAll: aSystemId;
			nextPut: $"]
	! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'monty 12/31/2013 00:38'!
test1attributeDeclarations
	self
		assertAttributeDecl: XMLAttributeDeclaration new
		hasElementName: ''
		attributeName: ''
		type: ''
		default: ''
		value: ''.
	self
		assertAttributeDecl:
			(XMLAttributeDeclaration
				element: 'testElement'
				attribute: 'testAttribute'
				type: 'CDATA'
				defaultPragma: 'IMPLIED'
				defaultValue: '')
		hasElementName: 'testElement'
		attributeName: 'testAttribute'
		type: 'CDATA'
		default: 'IMPLIED'
		value: ''.
	self
		assertAttributeDecl:
			(XMLAttributeDeclaration
				element: 'testElement'
				attribute: 'testAttribute'
				type: 'CDATA'
				defaultPragma: ''
				defaultValue:  'one')
		hasElementName: 'testElement'
		attributeName: 'testAttribute'
		type: 'CDATA'
		default: ''
		value: 'one'.! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'monty 12/21/2013 03:14'!
test2elementDeclarations
	self
		assertElementDecl: XMLElementDeclaration new
		hasName: ''
		contentModel: ''.
	self
		assertElementDecl:
			(XMLElementDeclaration
				element: 'testElement'
				contentModel: '(one,two)')
		hasName: 'testElement'
		contentModel: '(one,two)'.! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'monty 11/13/2013 21:24'!
test3entityDeclarations
	self
		assertEntityDecl: XMLGeneralEntityDeclaration new
		hasName: ''
		replacement: ''
		publicID: ''
		systemID: ''
		ndada: ''.
	self
		assertEntityDecl:
			(XMLGeneralEntityDeclaration
				name: 'testEntity'
				replacement: 'testReplacement')
		hasName: 'testEntity'
		replacement: 'testReplacement'
		publicID: ''
		systemID: ''
		ndada: ''.
	self
		assertEntityDecl:
			(XMLGeneralEntityDeclaration
				name: 'testEntity'
				publicID: '-//test//test-id//EN'
				systemID: '')
		hasName: 'testEntity'
		replacement: ''
		publicID: '-//test//test-id//EN'
		systemID: ''
		ndada: ''.
	self
		assertEntityDecl:
			(XMLGeneralEntityDeclaration
				name: 'testEntity'
				publicID: '-//test//test-id//EN'
				systemID: 'http://test')
		hasName: 'testEntity'
		replacement: ''
		publicID: '-//test//test-id//EN'
		systemID: 'http://test'
		ndada: ''.
	self
		assertEntityDecl:
			(XMLGeneralEntityDeclaration
				name: 'testEntity'
				publicID: ''
				systemID: 'http://test')
		hasName: 'testEntity'
		replacement: ''
		publicID: ''
		systemID: 'http://test'
		ndada: ''.
	self
		assertEntityDecl:
			(XMLGeneralEntityDeclaration
				name: 'testEntity'
				publicID: '-//test//test-id//EN'
				systemID: 'http://test'
				ndata: 'testNotation')
		hasName: 'testEntity'
		replacement: ''
		publicID: '-//test//test-id//EN'
		systemID: 'http://test'
		ndada: 'testNotation'.! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'JAAyer 11/26/2010 18:17'!
test4notationDeclarations
	self
		assertNotationDecl: XMLNotationDeclaration new
		hasName: ''
		publicID: ''
		systemID: ''.
	self
		assertNotationDecl:
			(XMLNotationDeclaration
				name: 'testNotation'
				publicID: '//test//'
				systemID: '')
		hasName: 'testNotation'
		publicID: '//test//'
		systemID: ''.
	self
		assertNotationDecl:
			(XMLNotationDeclaration
				name: 'testNotation'
				publicID: '//test//'
				systemID: 'http://test/')
		hasName: 'testNotation'
		publicID: '//test//'
		systemID: 'http://test/'.
	self
		assertNotationDecl:
			(XMLNotationDeclaration
				name: 'testNotation'
				publicID: ''
				systemID: 'http://test/')
		hasName: 'testNotation'
		publicID: ''
		systemID: 'http://test/'.! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 11/8/2010 19:19'!
assertNode: aNode hasChildren: aNodeCollection
	self assert: aNode nodes asArray = aNodeCollection asArray! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:28'!
assertNodeHasNoChildren: aNode
	self
		deny: aNode hasChildren;
		assert: aNode nodes isEmpty;
		assert: aNode firstNode isNil;
		assert: aNode lastNode isNil! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:32'!
should: aBlock addNode: aNode to: aParent
	self should: aBlock addNodes: (Array with: aNode) to: aParent! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:36'!
should: aBlock addNodes: aNodeCollection to: aParent
	| nodesAdded |

	self deny: (aNodeCollection anySatisfy: [:each | aParent includesNode: each]).
	(nodesAdded := aBlock value) isCollection
		ifFalse: [nodesAdded := Array with: nodesAdded].
	self
		assert: nodesAdded asArray = aNodeCollection asArray;
		assert:
			(nodesAdded allSatisfy: [:each |
				(aParent includesNode: each) and: [each parent == aParent]])! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:28'!
should: aBlock removeNode: aNode from: aParent
	self should: aBlock removeNodes: (Array with: aNode) from: aParent! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:32'!
should: aBlock removeNodes: aNodeCollection from: aParent
	| nodesRemoved |

	self assert: (aNodeCollection allSatisfy: [:each | aParent includesNode: each]).
	(nodesRemoved := aBlock value) isCollection
		ifFalse: [nodesRemoved := Array with: nodesRemoved].

	self
		assert: nodesRemoved asArray = aNodeCollection asArray;
		deny: (nodesRemoved anySatisfy: [:each | aParent includesNode: each])! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test1emptyNodeWithChildren
	self assertNodeHasNoChildren: XMLMockNodeWithChildren new! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/8/2010 19:19'!
test2addingNodes
	| node children |

	node := XMLMockNodeWithChildren new.
	children := self newNodes: 6.
	self
		should: [node addNode: children second] addNode: children second to: node;
		should: [node addNodeFirst: children first] addNode: children first to: node.
	self
		should: [node addNodes: (children copyFrom: 3 to: 4)]
		addNodes: (children copyFrom: 3 to: 4)
		to: node.
	self
		should: [node addNode: children sixth after: children fourth]
		addNode: children sixth
		to: node.
	self
		should: [node addNode: children fifth before: children sixth]
		addNode: children fifth
		to: node.
	self assertNode: node hasChildren: children.

	self
		should: [node addNode: (XMLMockNode new parent: XMLMockNode new)] raise: Error;
		should: [node addNode: children first] raise: Error.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/8/2010 19:20'!
test3removingNodes
	| node children |

	node := XMLMockNodeWithElements new.
	children := self newNodes: 5.
	node addNodes: children.
	self
		should: [node removeNode: children first]
		removeNode: children first
		from: node.
	self
		should: [node removeNodes: (children copyFrom: 2 to: 3)]
		removeNodes: (children copyFrom: 2 to: 3)
		from: node.
	self assert: node removeNodes == node.

	self assertNodeHasNoChildren: node.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'OttoBehrens 2/22/2011 23:42'!
test4accessingAdjacentNodes
	| node children |
	node := XMLMockNodeWithElements new.
	children := self newNodes: 5.
	node addNodes: children.
	children
		do: [ :each | 
			| next previous |
			next := children after: each ifAbsent: [ nil ].
			previous := children before: each ifAbsent: [ nil ].
			self
				assert: each nextNode == next;
				assert: each previousNode == previous;
				assert: (node nodeAfter: each) == next;
				assert: (node nodeBefore: each) == previous ].
	self
		assert: (node nodeAfter: XMLNode new) isNil;
		assert: (node nodeBefore: XMLNode new) isNil! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test5accessingFirstAndLastNodes
	| node children |

	node := XMLMockNodeWithElements new.
	self
		assert: node firstNode isNil;
		assert: node lastNode isNil.

	children := self newNodes: 3.
	node addNode: children first.
	self
		assert: node firstNode == children first;
		assert: node lastNode == children first.

	node addNodes: children allButFirst.
	self
		assert: node firstNode == children first;
		assert: node lastNode == children last.

	node removeNode: children first.
	self
		assert: node firstNode == children second;
		assert: node lastNode == children last.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test6replacingNodes
	| node children newChildren |

	node := XMLMockNodeWithChildren new.
	children := self newNodes: 3.
	newChildren := self newNodes: 3.
	node addNodes: children.
	self
		should: [node replaceNode: children first with: newChildren first]
		addNode: newChildren first
		to: node.
	self
		should: [node replaceNode: children second with: newChildren second]
		addNode: newChildren second
		to: node.
	self
		should: [node replaceNode: children third with: newChildren third]
		addNode: newChildren third
		to: node.

	self assert: node nodes asOrderedCollection = newChildren asOrderedCollection.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test7childNodeEnumeration
	| node nodes |

	node := XMLMockNodeWithElements new.
	self should: [:block | node nodesDo: block] enumerate: #().

	nodes := self newNodes: 5.
	node addNodes: nodes.
	self should: [:block | node nodesDo: block] enumerate: nodes.! !

XMLNodesWithChildrenTest subclass: #XMLNodesWithElementsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLNodesWithElementsTest subclass: #XMLElementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/26/2010 18:39'!
assertElement: anElement hasAttribute: anAttributeName equalTo: anAttributeValue
	self
		assert: (anElement includesAttribute: anAttributeName);
		assert: (anElement attributeAt: anAttributeName) = anAttributeValue;
		assert: (anElement attributeNodeAt: anAttributeName) notNil;
		assert: (anElement attributeNodeAt: anAttributeName) value = anAttributeValue! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:56'!
assertElement: anElement hasAttributes: aDictionary
	self assert:
		((aDictionary isMemberOf: Dictionary)
			ifTrue: [(Dictionary newFrom: anElement attributes) = aDictionary]
			ifFalse: [anElement attributes = aDictionary])! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:56'!
assertElement: anElement hasContentString: aContentString withStrings: aStringArray
	| combinedStrings |

	self assert: anElement contentString = aContentString.
	combinedStrings := anElement stringNodes collect: [:each | each string].
	self assert: combinedStrings asArray = aStringArray asArray.! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:56'!
assertElement: anElement hasName: aName attributes: aDictionary contentString: aString
	self
		assert: (anElement isNamed: aName);
		assertElement: anElement hasAttributes: aDictionary;
		assert: anElement contentString = aString! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:51'!
assertNodeHasNoContentNodes: aNode
	super assertNodeHasNoContentNodes: aNode.
	self assertNodeHasNoStringNodes: aNode.! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:50'!
assertNodeHasNoStringNodes: aNode
	self
		deny: aNode hasStringNodes;
		assert: aNode stringNodes isEmpty;
		assert: aNode strings isEmpty
	! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test1emptyElement
	self
		assertNode: XMLElement new hasTypes: #(isElement isContentNode);
		assertNodeHasNoChildren: XMLElement new! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test2elementNaming
	| element |

	element := XMLElement new.
	self
		assertNode: element hasName: '' namespaceURI: '';
		assertNode: (element name: 'testElement') hasName: 'testElement' namespaceURI: ''.
	self
		assertNode: (element name: 'testPrefix:testElement' namespaceURI: 'http://test/')
		hasName: 'testPrefix:testElement'
		namespaceURI: 'http://test/'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test3elementContentString
	| element str |

	str := self markupString.
	(element := XMLElement new) contentString: str.
	self
		assert: element hasChildren;
		assert: element hasContentNodes;
		assert: element hasStringNodes;
		assert: element contentString = str.

	"Intersperse between text"
	element addNode: XMLMockNode new.
	self
		assertElement: (element addContent: str)
		hasContentString: str, ' ', str
		withStrings: (Array with: str with: str).

	"addContent: should combine neighboring string nodes"
	self
		assertElement: (element addContent: (XMLString string: str))
		hasContentString: str, ' ', str, str
		withStrings: (Array with: str with: str, str).

	element addElement: ((XMLElement named: 'childElementWithContent') addContent: 'test content').
	self assert: (element contentStringAt: 'childElementWithContent') = 'test content'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 08:03'!
test4elementAttributes
	| element attrs |

	element := XMLElement new.
	self
		assert: (element attributeAt: 'foo') isEmpty;
		assert: (element attributeAt: 'foo' ifAbsent: ['absent']) = 'absent';
		deny: (element includesAttribute: 'foo').

	self
		assert: (element attributeAt: 'foo' put: 'test') = 'test';
		assert: element hasAttributes;
		assertElement: element hasAttribute: 'foo' equalTo: 'test';
		assert: (element attributeAt: 'bar' ifAbsentPut: ['test2']) = 'test2';
		assertElement: element hasAttribute: 'bar' equalTo: 'test2';
		assert: element attributeNames asArray =  #('foo' 'bar').

	self
		assert: element attributeAssociations asArray =
			(Array with: 'foo'->'test' with: 'bar'->'test2').
	self
		assertElement: element
		hasAttributes:
			(OrderPreservingDictionary
				with: 'foo'->'test'
				with: 'bar'->'test2').

	self
		assert: (element removeAttribute: 'foo') = 'test';
		assert: (element removeAttribute: 'bar') = 'test2';
		deny: element hasAttributes.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 08:03'!
test5elementConstructors
	| noAttributes attributes str element |

	noAttributes := OrderPreservingDictionary new.
	self
		assertElement: (XMLElement named: 'foo')
		hasName: 'foo'
		attributes: noAttributes
		contentString: ''.

	attributes := OrderPreservingDictionary with: 'foo'->'test1' with: 'bar'->'test2'.
	self
		assertElement: (XMLElement name: 'foo' attributes: attributes)
		hasName: 'foo'
		attributes: attributes
		contentString: ''.

	str := self markupString.
	self
		assertElement: (XMLElement named: 'foo')
		hasName: 'foo'
		attributes: noAttributes
		contentString: ''.
	self
		assertElement: (XMLElement name: 'foo' attributes: attributes)
		hasName: 'foo'
		attributes: attributes
		contentString: ''.

	element := XMLElement
		name: 'prefix:foo'
		namespaceURI: 'http://foo'
		attributes: attributes.
	self
		assertElement: element hasName: 'prefix:foo' attributes: attributes contentString: '';
		assertNode: element hasPrefix: 'prefix' uri: 'http://foo'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 08:03'!
test6elementPrinting
	| element attributes |

	element := (XMLElement named: 'element').
	attributes := OrderPreservingDictionary with: 'b'->'two' with: 'a'->'one'.
	self
		assert: element printString = '<element />';
		assert: (element contentString: 'test') printString = '<element>test</element>';
		assert: (element setAttributes: attributes) printString = '<element b="two" a="one">test</element>'! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test7contentNodeEnumeration
	| node nodes elements stringNodes contentNodes |

	node := XMLElement new.
	elements := self newElements: 3.
	stringNodes := self newStrings: 3.
	contentNodes := self intersperse: elements with: stringNodes.
	node addNodes: (self intersperseWithNodes: contentNodes).

	self
		assert: node contentNodes = contentNodes;
		should: [:block | node contentNodesDo: block] enumerate: contentNodes;
		should: [:block | node stringNodesDo: block] enumerate: stringNodes.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/12/2014 01:06'!
testCanonicalPrinting
	| element |

	element :=
		XMLElement
			name: 'test'
			namespaces: (Array with: 'b' -> 'ns2' with: 'a' -> 'ns1' with: '' -> 'default')
			attributes: (Array with: 'c' -> '3' with: 'b' -> '2' with: 'a' -> '1').

	self assert:
		element printString =
			'<test xmlns:b="ns2" xmlns:a="ns1" xmlns="default" c="3" b="2" a="1" />'.
	self assert:
		element canonicallyPrinted =
			'<test xmlns="default" xmlns:a="ns1" xmlns:b="ns2" a="1" b="2" c="3"></test>'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/13/2014 01:49'!
testPrintingDefaultNamespaces
	| element |

	element :=
		XMLElement
			name: 'element'
			namespaceURI: 'default'.
	element addElement:
		((XMLElement
			name: 'element'
			namespaceURI: '')
			addElement: (XMLElement name: 'empty');
			yourself).

	self assert:
		element printString =
			('<element xmlns="default">',
			'<element xmlns=""><empty /></element>',
			'</element>').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/13/2014 01:46'!
testPrintingNonRootNamespaces
	| element |

	element :=
		XMLElement
			name: 'element'.
	element addElement:
		(XMLElement
			name: 'used:empty'
			namespaces: (Array with: '' -> 'defaultNS' with: 'used' -> 'usedNS')).

	self assert:
		element printString =
			('<element>',
			'<used:empty xmlns="defaultNS" xmlns:used="usedNS" />',
			'</element>').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/13/2014 01:47'!
testPrintingRootNamespaces
	| element |

	element :=
		XMLElement
			name: 'used:element'
			namespaces: (Array with: '' -> 'defaultNS' with: 'used' -> 'usedNS').
	element addElement: (XMLElement name: 'empty').

	self assert:
		element printString =
			('<used:element xmlns="defaultNS" xmlns:used="usedNS">',
			'<empty />',
			'</used:element>').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/12/2014 01:13'!
testUndeclaringAttributePrefixes
	| element |

	element :=
		XMLElement
			name: 'element'
			namespaces: (Array with: '' -> 'defaultNS' with: 'unused' -> 'unusedNS' with: 'used' -> 'usedNS')
			attributes: (Array with: 'name' -> 'value' with: 'used:name' -> 'value').

	self
		shouldnt: [element undeclarePrefix: '']
		raise: XMLNamespaceException.
	self
		shouldnt: [element undeclarePrefix: 'unused']
		raise: XMLNamespaceException.
	self
		should: [element undeclarePrefix: 'used']
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/12/2014 01:13'!
testUndeclaringElementPrefixes
	| element |

	element :=
		XMLElement
			name: 'used:element'
			namespaces: (Array with: '' -> 'defaultNS' with: 'unused' -> 'unusedNS' with: 'used' -> 'usedNS').

	self
		shouldnt: [element undeclarePrefix: '']
		raise: XMLNamespaceException.
	self
		shouldnt: [element undeclarePrefix: 'unused']
		raise: XMLNamespaceException.
	self
		should: [element undeclarePrefix: 'used']
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/12/2014 01:39'!
testUndeclaringReservedPrefixes
	| element |

	element :=
		XMLElement
			name: 'element'
			namespaces: (Array with: 'xml' -> XMLNamespaceScope xmlNamespaceURI).

	self
		shouldnt: [element undeclarePrefix: 'xml']
		raise: XMLNamespaceException.
	self
		should: [element undeclarePrefix: 'xml']
		raise: XMLNamespaceException.
	self
		should: [element undeclarePrefix: 'xmlns']
		raise: XMLNamespaceException.! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:58'!
assertDocument: aDocument hasDoctype: aDoctype version: aVersion encoding: anEncoding
	self
		assert: aDocument doctypeDeclaration = aDoctype;
		assert: aDocument version = aVersion;
		assert: aDocument encoding = anEncoding! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/8/2010 19:21'!
assertNode: aNode hasElements: anElementCollection
	self assert: aNode elements asArray = anElementCollection asArray! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:48'!
assertNodeHasNoChildren: aNode
	super assertNodeHasNoChildren: aNode.
	self assertNodeHasNoContentNodes: aNode! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:51'!
assertNodeHasNoContentNodes: aNode
	self
		deny: aNode hasContentNodes;
		assert: aNode contentNodes isEmpty;
		assert: aNode contentString isEmpty;
		assertNodeHasNoElements: aNode! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 23:38'!
assertNodeHasNoElements: aNode
	self
		deny: aNode hasElements;
		assert: aNode elements isEmpty;
		assert: aNode firstElement isNil;
		assert: aNode lastElement isNil! !

!XMLNodesWithElementsTest methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:03'!
newElements: aNumber
	^ self
		createNodes: aNumber
		with: [:i | XMLElement named: 'testElement', i printString]! !

!XMLNodesWithElementsTest methodsFor: 'accessing' stamp: 'monty 2/1/2014 06:52'!
newElementsNamed: aNameArray
	| names qualifiedName element |

	names := aNameArray asOrderedCollection.
	^ XMLOrderedList newFrom:
		(self
			createNodes: aNameArray size
			with: [
				((qualifiedName := names removeFirst) includes: $:)
					ifTrue: [
						element := XMLElement
							name: qualifiedName
							namespaceURI: 'http://', (qualifiedName qualifiedNamePrefix)]
					ifFalse: [element := XMLElement named: qualifiedName].
				element]).! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:54'!
should: aBlock addElement: anElement to: aParent
	self should: aBlock addElements: (Array with: anElement) to: aParent! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/5/2010 22:47'!
should: aBlock addElements: anElementCollection to: aParent
	| elementsAdded |

	self deny: (anElementCollection anySatisfy: [:each | aParent elements includes: each]).
	self
		should: [elementsAdded := aBlock value]
		addNodes: anElementCollection
		to: aParent.
	elementsAdded isCollection
		ifFalse: [elementsAdded := Array with: elementsAdded].
	self
		assert: elementsAdded asArray = anElementCollection asArray;
		assert: (elementsAdded allSatisfy: [:each | aParent elements includes: each]).! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:54'!
should: aBlock removeElement: anElement from: aParent
	self should: aBlock removeElements: (Array with: anElement) from: aParent! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/5/2010 22:48'!
should: aBlock removeElements: anElementCollection from: aParent
	| elementsRemoved |

	self assert: (anElementCollection allSatisfy: [:each | aParent elements includes: each]).
	self
		should: [elementsRemoved := aBlock value]
		removeNodes: anElementCollection
		from: aParent.
	elementsRemoved isCollection
		ifFalse: [elementsRemoved := Array with: elementsRemoved].
	self
		assert: elementsRemoved asArray = anElementCollection asArray;
		deny: (elementsRemoved anySatisfy: [:each | aParent elements includes: each]).! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test01emptyNodeWithElements
	self assertNodeHasNoChildren: XMLMockNodeWithElements new! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test02addingElements
	| node nodes elements children |

	node := XMLMockNodeWithElements new.
	elements := self newElements: 6.
	nodes := self newNodes: 6.
	children := self intersperse: elements with: nodes.

	self
		should: [node addNodes: (elements copyFrom: 2 to: 3)]
		addElements: (elements copyFrom: 2 to: 3)
		to: node.
	self
		should: [node addNode: nodes second after: elements second]
		addNode: nodes second
		to: node.
	self
		should: [node addNode: nodes third after: elements third]
		addNode: nodes third
		to: node.
	self
		should: [node addNodeFirst: elements first]
		addElement: elements first
		to: node.
	self
		should: [node addNode: nodes first before: elements second]
		addNode: nodes first
		to: node.
	self
		should: [node addElement: elements fourth. elements fourth]
		addElement: elements fourth
		to: node.
	self
		should: [node addNode: nodes fourth after: elements fourth]
		addNode: nodes fourth
		to: node.
	self
		should: [node addNode: elements fifth after: nodes fourth]
		addElement: elements fifth
		to: node.
	self
		should: [node addNodes: (nodes allButFirst: 4)]
		addNodes: (nodes allButFirst: 4)
		to: node.
	self
		should: [node addNode: (elements sixth) before: nodes sixth]
		addElement: elements sixth
		to: node.

	self
		assertNode: node hasChildren: children;
		assertNode: node hasElements: elements.

	self
		should: [node addNode: (XMLElement new parent: XMLMockNode new)] raise: Error;
		should: [node addNode: elements first] raise: Error.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test03removingElements
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('testOne' 'testPrefix:testTwo' 'testTwo' 'testThree').
	node addNodes: (self intersperseWithNodes: elements).
	self
		assert: (node removeNode: elements first) == elements first;
		assert: node elements = elements allButFirst;
		assert: (node elementAt: 'testOne') isNil;
		assert: (node elementsAt: 'testOne') isEmpty;
		assert: (node removeNode: elements second) = elements second;
		assert: node elements = (elements last: 2);
		assert: (node elementAt: 'testTwo' namespaceURI: 'http://testPrefix') isNil;
		assert: (node elementsAt: 'testTwo' namespaceURI: 'http://testPrefix') isEmpty;
		assert: (node elementAt: 'testTwo') == elements third;
		assert: (node elementsAt: 'testTwo') = (XMLOrderedList with: elements third);
		assert: (node removeNode: elements third) == elements third;
		assert: node elements = (elements last: 1);
		assert: (node elementAt: 'bar') isNil;
		assert: (node elementsAt: 'bar') isEmpty;
		assert: node firstElement = elements last;
		assert: (node removeNode: elements last) = elements last;
		deny: node hasElements;
		assert: node firstElement isNil.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test04accessingAdjacentElements
	| node elements |

	node := XMLMockNodeWithElements new.
	elements := self newElements: 3.
	node addNodes: (self intersperseWithNodes: elements).
	elements do: [:each | | next previous |
		next := elements after: each ifAbsent: [nil].
		previous := elements before: each ifAbsent: [nil].
		
		self
			assert: each nextElement == next;
			assert: each previousElement == previous;
			assert: (node elementAfter: each) == next;
			assert: (node elementBefore: each) == previous].

	self
		assert: (node elementAfter: XMLElement new) isNil;
		assert: (node elementBefore: XMLElement new) isNil.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test05accessingFirstAndLastElements
	| node elements |

	node := XMLMockNodeWithElements new.
	self
		assert: node firstElement isNil;
		assert: node lastElement isNil.

	elements := self newElements: 3.
	node addNode: elements first.
	self
		assert: node firstElement == elements first;
		assert: node lastElement == elements first.

	node addNodes: elements allButFirst.
	self
		assert: node firstElement == elements first;
		assert: node lastElement == elements last.

	node removeNode: elements first.
	self
		assert: node firstElement == elements second;
		assert: node lastElement == elements last.

	node removeNodes.
	node addNodes: (self intersperseWithNodes: elements).
	self
		assert: node firstElement == elements first;
		assert: node lastElement == elements last.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test06chlidNodeAndElementEnumeration
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	self
		should: [:block | node nodesDo: block] enumerate: #();
		should: [:block | node elementsDo: block] enumerate: #().

	elements := self newElements: 3.
	nodes := self intersperseWithNodes: elements.
	node addNodes: nodes.
	self
		should: [:block | node nodesDo: block] enumerate: nodes;
		should: [:block | node elementsDo: block] enumerate: elements.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/26/2010 12:05'!
test07elementAccessing
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('testOne' 'testTwo' 'testPrefix:testOne' 'testThree').
	node addNodes: (self intersperseWithNodes: elements).
	self
		assert: (node elementAt: 'absent') isNil;
		assert: (node elementAt: 'absent' ifAbsent: ['absent']) = 'absent';
		assert: (node elementsAt: 'absent') isEmpty;
		assert: (node elementAt: 'testOne') == elements first;
		assert: (node elementAt: 'testTwo') == elements second;
		assert: (node elementsAt: 'testOne') = ((elements first: 3) copyWithout: elements second);
		assert: (node elementAt: 'testPrefix:testOne') == elements third;
		assert: (node elementsAt: 'testPrefix:testOne') = (XMLOrderedList with: elements third);
		assert: (node elementAt: 'testOne' namespaceURI: 'absent') isNil;
		assert: (node elementAt: 'absent' namespaceURI: 'http://testPrefix') isNil;
		assert: (node elementAt: 'testOne' namespaceURI: 'absent' ifAbsent: ['absent']) = 'absent';
		assert: (node elementAt: 'absent' namespaceURI: 'http://testPrefix' ifAbsent: ['absent']) = 'absent';
		assert: (node elementAt: 'testOne' namespaceURI: 'http://testPrefix') == elements third;
		assert: (node elementsAt: 'testOne' namespaceURI: 'absent') isEmpty;
		assert: (node elementsAt: 'absent' namespaceURI: 'http://testPrefix') isEmpty;
		assert: (node elementsAt: 'testOne' namespaceURI: 'http://testPrefix') = (elements copyFrom: 3 to: 3)! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/26/2010 12:07'!
test08firstTagSearching
	| node elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('foo' 'bar' 'prefix:foo' 'prefix:bar' 'bar' 'baz').
	node addNodes: (elements first: 2).
	node nodes first addNodes: (elements copyFrom: 3 to: 4).
	node nodes second addNodes: (elements copyFrom: 5 to: 6).
	self
		assert: (node findElementNamed: 'absent') isNil;
		assert: (node findElementNamed: 'foo' with: [:each | false]) isNil.

	self
		assert: (node findElementNamed: 'foo') = elements first;
		assert: (node findElementNamed: 'prefix:bar') = elements fourth;
		assert: (elements first findElementNamed: 'foo') = elements first.
	self assert: (node
		findElementNamed: 'bar'
		with: [:each | each = elements fifth]) = elements fifth.
	self assert: (elements first
		findElementNamed: 'foo'
		with: [:each | each = elements third]) = elements third.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/26/2010 12:08'!
test09tagsNamedSearching
	| node elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('foo' 'bar' 'prefix:foo' 'bar' 'prefix:bar' 'baz').
	node addNodes: (elements first: 2).
	node nodes first addNodes: (elements copyFrom: 3 to: 4).
	node nodes second addNodes: (elements copyFrom: 5 to: 6).
	self
		should: [:block | node allElementsNamed: 'absent' do: block] enumerate: #();
		should: [:block | node elementsAt: 'absent' do: block] enumerate: #().

	self
		should: [:block | node allElementsNamed: 'foo' do: block]
		enumerate: (Array with: elements first with: elements third).
	self
		should: [:block | node allElementsNamed: 'prefix:foo' do: block]
		enumerate: (Array with: elements third).
	self
		should: [:block | node firstElement elementsAt: 'foo' do: block]
		enumerate: (Array with: elements third).! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test10elementRenaming
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('foo' 'prefix:bar' 'bar' 'baz').
	node addNodes: (self intersperseWithNodes: elements).

	elements first name: 'notfoo'.
	self
		deny: (node includesElement: 'foo');
		assert: (node elementAt: 'foo') isNil;
		assert: (node elementsAt: 'foo') isEmpty;
		assert: (node elementAt: 'notfoo') = elements first.

	elements third name: 'prefix:bar' namespaceURI: 'http://bar'.
	self assert: (node elementsAt: 'prefix:bar') = (elements copyFrom: 2 to: 3).

	elements fourth name: 'bar'.
	self assert: (node elementsAt: 'bar') = elements allButFirst.

	elements fourth name: 'foo'.
	self assert: (node elementAt: 'foo') = elements last.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test11documentTest
	| doc root doctype |

	doc := XMLDocument new.
	root := XMLElement new.
	doctype := XMLDoctypeDeclaration root: 'root' publicID: 'public' systemID: 'system'.
	self
		assert: doc isDocument;
		assert: doc root isNil;
		assert: (doc addElement: root) root = root;
		assertDocument: doc hasDoctype: nil version: nil encoding: 'UTF-8'.

	self
		assertDocument: (doc doctypeDeclaration: doctype)
		hasDoctype: doctype
		version: nil
		encoding: 'UTF-8'.
	self
		assertDocument: (doc version: 1.0)
		hasDoctype: doctype
		version: 1.0
		encoding: 'UTF-8'.
	self
		assertDocument: (doc encoding: 'ISO Latin-1')
		hasDoctype: doctype
		version: 1.0
		encoding: 'ISO Latin-1'.! !

XMLNodeTest subclass: #XMLNodesWithoutChildrenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 21:02'!
assertAttribute: anAttribute hasName: aName value: aValue
	self
		assert: anAttribute name = aName;
		assert: (anAttribute isNamed: aName);
		assert: anAttribute value = aValue;
		assert: anAttribute printString = (aName, '="', aValue, '"')! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 16:30'!
assertCData: aCDataNode hasString: aString
	self
		assertStringNode: aCDataNode
		hasString: aString
		printString: '<!![CDATA[', aString, ']]>'! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 06:21'!
assertComment: aComment hasString: aString
	self
		assert: aComment string = aString;
		assert: aComment printString = ('<!!--', aString, '-->')! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:29'!
assertNode: anObservedNode canNotifyNodes: anObserverArray
	anObservedNode notifyOf: #mockNotifyWith: with: #one.
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one)]).

	anObservedNode notifyOf: #mockNotifyWith:with: with: #one with: #two.
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one two)]).

	anObservedNode notifyOf: #mockNotifyWith:with:with: with: #one with: #two with: #three.
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one two three)]).

	anObserverArray do: [:each | each lastNotificationArguments: nil].
	anObservedNode notifyOf: #mockNotifyWith:with:with: withArguments: #(one two three).
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one two three)]). ! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:19'!
assertPI: aNode hasTarget: aTarget data: aData
	self
		assert: aNode target = aTarget;
		assert: aNode data = aData;
		assert: aNode printString = ('<?', aTarget, ' ', aData, '?>')! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 00:30'!
assertStringNode: aStringNode hasString: aString printString: aPrintString
	self
		assert: aStringNode string = aString;
		assert: aStringNode contentString = aString;
		assert: aStringNode printString = aPrintString! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/15/2010 05:02'!
test1basicNodeProtocol
	| node parent |

	node := XMLMockNode new.
	self
		assertNode: node hasType: nil;
		deny: node hasChildren;
		assert: (#(nextNode previousNode parent) allSatisfy: [:each | (node perform: each) isNil]);
		assert: node printString = (node class name asString, '(', node identityHash printString, ') ');
		assert: node contentString isEmpty.

	parent := XMLMockNode new.
	node parent: parent.
	self
		assert: node hasParent;
		assert: node parent == parent.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:05'!
test2nodeNotification
	| observedNode firstObserverNode secondObserverNode |

	observedNode := XMLMockNode new.
	self
		shouldnt: [observedNode notifyOf: #invalid: with: #one] raise: Error;
		shouldnt: [observedNode notifyOf: #invalid:invalid: with: #one with: #two] raise: Error;
		shouldnt: [observedNode notifyOf: #invalid:invalid:invalid: with: #one with: #two with: #three] raise: Error;
		shouldnt: [observedNode notifyOf: #invalid:invalid:invalid: withArguments: #(one two three)] raise: Error.

	firstObserverNode := XMLMockObserverNode new.
	observedNode addObserver: firstObserverNode.
	self assertNode: observedNode canNotifyNodes: (Array with: firstObserverNode).

	secondObserverNode := XMLMockObserverNode new.
	observedNode addObserver: secondObserverNode.
	self
		assertNode: observedNode
		canNotifyNodes: (Array with: firstObserverNode with: secondObserverNode).

	observedNode removeObserver: firstObserverNode.
	self assertNode: observedNode canNotifyNodes: (Array with: secondObserverNode).

	self
		should: [observedNode notifyOf: #invalid: with: #one] raise: Error;
		should: [observedNode notifyOf: #invalid:invalid: with: #one with: #two] raise: Error;
		should: [observedNode notifyOf: #invalid:invalid:invalid: with: #one with: #two with: #three] raise: Error;
		should: [observedNode notifyOf: #invalid:invalid:invalid: withArguments: #(one two three)] raise: Error.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test3processingInstructionNodes
	| pi t d |

	pi := XMLPI new.
	t := 'target'.
	d := 'data'.
	self
		assertNode: pi hasType: #isPI;
		assertPI: pi hasTarget: '' data: '';
		assertPI: (pi target: t) hasTarget: t data: '';
		assertPI: (pi data: d) hasTarget: t data: d;
		assertPI: (XMLPI target: t data: d) hasTarget: t data: d.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test4stringNodes
	| node raw escaped |

	node := XMLString new.
	raw := self markupString.
	escaped := self escapedString.
	self
		assertNode: node hasTypes: #(isStringNode isContentNode);
		assertStringNode: node hasString: '' printString: '';
		assertStringNode: (node string: raw) hasString: raw printString: escaped;
		assertStringNode: (node addContent: raw) hasString: raw, raw printString: escaped, escaped.
	self
		assertStringNode: (node addContent: (XMLString string: raw))
		hasString: raw, raw, raw
		printString: escaped, escaped, escaped.

	self
		assertStringNode: (XMLString string: raw)
		hasString: raw
		printString: escaped.
	self
		assertStringNode: (XMLString string: (XMLString string: raw))
		hasString: raw
		printString: escaped.

	self
		assertStringNode: raw asXMLString hasString: raw printString: escaped;
		assert: node asXMLString == node.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:27'!
test5cdataNode
	| node raw |

	node := XMLCData new.
	raw := self markupString.
	self
		assertNode: node hasTypes: #(isCData isStringNode isContentNode);
		assertCData: node hasString: '';
		assertCData: (node string: raw) hasString: raw;
		assertCData: (node addContent: raw) hasString: raw, raw;
		assertCData: (node addContent: raw asXMLString) hasString: raw, raw, raw;
		assert: node asXMLString class == XMLString;
		assert: node asXMLString string = node string.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test6commentNodes
	| comment raw escaped |

	comment := XMLComment new.
	self
		assertNode: comment hasType: #isComment;
		assertComment: comment hasString: '';
		assertComment: (comment string: 'test') hasString: 'test';
		assertComment: (XMLComment string: 'test') hasString: 'test'.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/24/2010 19:47'!
test7attributeNodes
	| attribute element |

	attribute := XMLAttribute new.
	self
		assertAttribute: attribute hasName: '' value: '';
		assertNode: attribute hasName: '' namespaceURI: '';
		assertAttribute: (attribute name: 'name') hasName: 'name' value: '';
		assertNode: attribute hasName: 'name' namespaceURI: '';
		assertAttribute: (attribute value: 'value') hasName: 'name' value: 'value';
		assertAttribute: (XMLAttribute name: 'name' value: 'value') hasName: 'name' value: 'value';
		assertNode: (attribute prefix: 'prefix') hasName: 'prefix:name' namespaceURI: '';
		assertNode: (XMLAttribute name: 'prefix:name' value: 'value') hasName: 'prefix:name' namespaceURI: ''.

	element := XMLElement new.
	self
		assert: attribute element isNil;
		assert: (attribute element: element) element == element.! !

TestCase subclass: #XMLOrderedListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLOrderedListTest subclass: #XMLObservableListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLObservableListTest class methodsFor: 'testing' stamp: 'JAA 3/26/2011 19:00'!
shouldInheritSelectors
	^ true! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 19:46'!
afterAddingToNotification
	^ #added:to:! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 19:46'!
afterRemovingFromNotification
	^ #removed:from:! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/12/2014 18:34'!
assertObserver: anObserver receivedNotificationOfAdding: anElement to: aList
	| firstNotification secondNotification |

	firstNotification := anObserver notifications removeFirst.
	secondNotification := anObserver notifications removeFirst.
	self
		assert:firstNotification selector = self beforeAddingToNotification;
		assert: firstNotification arguments = (Array with: anElement with: aList);
		assert: secondNotification selector = self afterAddingToNotification;
		assert: secondNotification arguments = (Array with: anElement with: aList)! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/12/2014 18:34'!
assertObserver: anObserver receivedNotificationOfRemoving: anElement from: aList
	| notification |

	notification := anObserver notifications removeFirst.
	self
		assert: notification selector = self afterRemovingFromNotification;
		assert: notification arguments = (Array with: anElement with: aList)! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 19:30'!
beforeAddingToNotification
	^ #adding:to:! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 19:01'!
listClass
	^ XMLObservableList! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/12/2014 20:07'!
should: aBlock add: anElement toAndNotifyOf: aList
	self
		should: aBlock
		addAll: (Array with: anElement)
		toAndNotifyOf: aList! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/12/2014 20:07'!
should: aBlock addAll: anElementCollection toAndNotifyOf: aList
	| observer |

	observer := XMLStubListObserver new.
	aList addObserver: observer.
	aBlock value.
	anElementCollection do: [:each |
		self assert: (aList includes: each).
		self 
			assertObserver: observer
			receivedNotificationOfAdding: each
			to: aList].! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/12/2014 20:17'!
should: aBlock remove: anElement fromAndNotifyOf: aList
	self
		should: aBlock
		removeAll: (Array with: anElement)
		fromAndNotifyOf: aList! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/12/2014 20:16'!
should: aBlock removeAll: anElementCollection fromAndNotifyOf: aList
	| observer |

	observer := XMLStubListObserver new.
	aList addObserver: observer.
	aBlock value.
	anElementCollection do: [:each |
		self deny: (aList includes: each).
		self
			assertObserver: observer
			receivedNotificationOfRemoving: each
			from: aList].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:55'!
testAdd
	| list |

	super testAdd.

	list := self emptyList.
	self
		should: [list add: self singleElement]
		add: self singleElement
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:37'!
testAddAfter
	| list |

	super testAddAfter.

	list := self listWithSingleElement.
	self
		should: [list add: self absentElement after: self singleElement]
		add: self absentElement
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:37'!
testAddAfterIndex
	| list |

	super testAddAfterIndex.

	list := self listWithSingleElement.
	self
		should: [list add: self absentElement afterIndex: (list indexOf: self singleElement)]
		add: self absentElement 
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:11'!
testAddAll
	| list |

	super testAddAll.
	list := self emptyList.
	self
		should: [list addAll: self listElements]
		addAll: self listElements
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:13'!
testAddAllFirst
	| list |

	super testAddAllFirst.
	list := self emptyList.
	self
		should: [list addAllFirst: self listElements]
		addAll: self listElements reversed
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:13'!
testAddAllLast
	| list |

	super testAddAllLast.
	list := self emptyList.
	self
		should: [list addAllLast: self listElements]
		addAll: self listElements
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:37'!
testAddBefore
	| list |

	super testAddBefore.

	list := self listWithSingleElement.
	self
		should: [list add: self absentElement before: self singleElement]
		add: self absentElement 
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:37'!
testAddBeforeIndex
	| list |

	super testAddBeforeIndex.

	list := self listWithSingleElement.
	self
		should: [list add: self absentElement beforeIndex: (list indexOf: self singleElement)]
		add: self absentElement 
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:37'!
testAddFirst
	| list |

	super testAddFirst.

	list := self listWithSingleElement.
	self
		should: [list addFirst: self absentElement]
		add: self absentElement
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:37'!
testAddLast
	| list |

	super testAddLast.

	list := self listWithSingleElement.
	self
		should: [list addLast: self absentElement]
		add: self absentElement
		toAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 18:34'!
testAtPut
	| list observer |

	super testAtPut.

	list := self listWithSingleElement.
	observer := XMLStubListObserver new.
	list addObserver: observer.
	list at: 1 put: self absentElement.
	self
		assertObserver: observer
		receivedNotificationOfRemoving: self singleElement
		from: list.
	self
		assertObserver: observer
		receivedNotificationOfAdding: self absentElement
		to: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:18'!
testRemoveAll
	| list |

	super testRemoveAll.
	list := self listWithElements.
	self
		should: [list removeAll]
		removeAll: self listElements reversed
		fromAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:52'!
testRemoveAt
	| list observer |

	super testRemoveAt.

	list := self listWithSingleElement.
	self
		should: [list removeAt: 1]
		remove: self singleElement
		fromAndNotifyOf: list.

	observer := XMLStubListObserver new.
	list addObserver: observer.
	self
		should: [list removeAt: 1] raise: Error;
		assert: observer notifications isEmpty.
	! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:53'!
testRemoveFirst
	| list observer |

	super testRemoveFirst.

	list := self listWithSingleElement.
	self
		should: [list removeFirst]
		remove: self singleElement
		fromAndNotifyOf: list.

	observer := XMLStubListObserver new.
	list addObserver: observer.
	self
		should: [list removeFirst] raise: Error;
		assert: observer notifications isEmpty.
	! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:41'!
testRemoveFirstFew
	| list  |

	super testRemoveFirstFew.
	
	list := self listWithElements.
	self
		should: [list removeFirst: 2]
		removeAll: (self listElements first: 2)
		fromAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:51'!
testRemoveIfAbsent
	| list observer |

	super testRemoveIfAbsent.

	list := self listWithSingleElement.
	self
		should: [list remove: self singleElement ifAbsent: [nil]]
		remove: self singleElement
		fromAndNotifyOf: list.

	observer := XMLStubListObserver new.
	list addObserver: observer.
	self
		assert: (list remove: self singleElement ifAbsent: [nil]) isNil;
		assert: observer notifications isEmpty.
	! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 19:53'!
testRemoveLast
	| list observer |

	super testRemoveLast.

	list := self listWithSingleElement.
	self
		should: [list removeLast]
		remove: self singleElement
		fromAndNotifyOf: list.

	observer := XMLStubListObserver new.
	list addObserver: observer.
	self
		should: [list removeLast] raise: Error;
		assert: observer notifications isEmpty.
	! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:42'!
testRemoveLastFew
	| list  |

	super testRemoveLastFew.
	
	list := self listWithElements.
	self
		should: [list removeLast: 2]
		removeAll: (self listElements last: 2) reversed
		fromAndNotifyOf: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 21:14'!
testReplaceFromToWithStartingAt
	| list observer |

	super testReplaceFromToWithStartingAt.

	list := self listWithElements.
	observer := XMLStubListObserver new.
	list addObserver: observer.
	list
		replaceFrom: 2
		to: 3
		with: #('new1' 'new2' 'new3')
		startingAt: 2.
	
	self
		assertObserver: observer
		receivedNotificationOfRemoving: (self listElements at: 2)
		from: list.
	self
		assertObserver: observer
		receivedNotificationOfAdding: 'new2'
		to: list.
	self
		assertObserver: observer
		receivedNotificationOfRemoving: (self listElements at: 3)
		from: list.
	self
		assertObserver: observer
		receivedNotificationOfAdding: 'new3'
		to: list.! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
absentElement
	^ 'absent'! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 9/28/2014 23:33'!
assertList: aList copiedFrom: aSecondList equals: aThirdList
	self
		assert: aList = aThirdList;
		deny: aList == aSecondList;
		deny: aList collection == aSecondList collection! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 9/28/2014 23:29'!
assertList: aList isCopyOf: aSecondList
	self
		assert: aList = aSecondList;
		deny: aList == aSecondList;
		deny: aList collection == aSecondList collection! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:46'!
defaultValue
	^ nil! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:19'!
emptyList
	^ self listClass new! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
listClass
	^ XMLOrderedList! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
listElements
	^ #('one' 'two' 'three' 'four' 'five')! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 18:15'!
listSpecies
	^ XMLOrderedList! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:24'!
listWithElements
	^ self listClass new
		addAll: self listElements;
		yourself! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:26'!
listWithSingleElement
	^ self listClass new
		add: self singleElement;
		yourself! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'JAA 3/26/2011 17:37'!
should: aBlock enumerate: aCollection
	| enumerated |

	enumerated := OrderedCollection new.
	aBlock value: [:each | enumerated add: each].
	self assert: enumerated = aCollection asOrderedCollection.! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
singleElement
	^ self listElements first! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:24'!
testAdd
	| list |

	list := self emptyList.
	self
		deny: list = self listWithElements;
		assert: (self listElements allSatisfy: [:each | (list add: each) = each]);
		assert: (list = self listWithElements).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:40'!
testAddAfter
	| list |

	list := self listWithSingleElement.
	self
		assert:
			(self listElements allButFirst reversed
				allSatisfy: [:each | (list add: each after: self singleElement) = each]);
		assert: list = self listWithElements;
		should: [list add: self singleElement after: self absentElement] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:39'!
testAddAfterIndex
	| list |

	list := self listWithSingleElement.
	self
		assert:
			(self listElements allButFirst reversed
				allSatisfy: [:each | (list add: each afterIndex: 1) = each]);
		assert: list = self listWithElements;
		should: [list add: self singleElement afterIndex: self listElements size + 1] raise: Error;
		should: [list add: self singleElement afterIndex: -1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:39'!
testAddAll
	| list |

	list := self emptyList.
	self
		deny: list = self listWithElements;
		assert: (list addAll: self listElements) = self listElements;
		assert: list = self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:41'!
testAddAllFirst
	| list |

	(list := self emptyList) add: self listElements last.
	self
		assert: (list addAllFirst: self listElements allButLast) = self listElements allButLast;
		assert: list = self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:42'!
testAddAllLast
	| list |

	list := self listWithSingleElement.
	self
		assert: (list addAllLast: self listElements allButFirst) = self listElements allButFirst;
		assert: list = self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:35'!
testAddBefore
	| list |

	(list := self emptyList) add: self listElements last.
	self
		assert:
			(self listElements allButLast
				allSatisfy: [:each | (list add: each before: self listElements last) = each]);
		assert: list = self listWithElements;
		should: [list add: self singleElement before: self absentElement] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:38'!
testAddBeforeIndex
	| list i |

	(list := self emptyList) add: self listElements last.
	i := 0.
	self
		assert:
			(self listElements allButLast
				allSatisfy: [:each |
					i := i + 1.
					(list add: each beforeIndex: i) = each]);
		assert: list = self listWithElements;
		should: [list add: self singleElement beforeIndex: self listElements size + 2] raise: Error;
		should: [list add: self singleElement beforeIndex: 0] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:47'!
testAddFirst
	| list |

	list := self emptyList.
	self
		assert: (self listElements reversed allSatisfy: [:each | (list addFirst: each) = each]);
		assert: list = self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:47'!
testAddLast
	| list |

	list := self emptyList.
	self
		assert: (self listElements allSatisfy: [:each | (list addLast: each) = each]);
		assert: list = self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:55'!
testAfter
	| list |

	list := self listWithElements.
	self
		assert:
			(self listElements allSatisfy: [:each |
				(list after: each) = (self listElements after: each ifAbsent: [self defaultValue])]).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:48'!
testAsArray
	self
		assert: self emptyList asArray = #();
		assert: self listWithElements asArray = self listElements asArray! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 00:47'!
testAt
	| list |

	list := self listWithElements.
	self
		assert: ((1 to: list size) allSatisfy: [:i | (list at: i) = (self listElements at: i)]);
		assert: (list at: 0) = self defaultValue;
		assert: (list at: list size + 1) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 14:46'!
testAtLast
	self
		assert: (self emptyList atLast: 1) = self defaultValue;
		assert: (self listWithSingleElement atLast: 1) = self singleElement;
		assert: (self listWithElements atLast: 2) = (self listElements atLast: 2);
		assert: (self listWithElements atLast: self listWithElements size + 1) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 14:43'!
testAtPut
	| list |

	list := self listWithElements.
	self
		assert:
			((1 to: list size) allSatisfy: [:i | | replacement |
				replacement := self listElements reversed at: i.
				(list at: i put: replacement) = replacement]);
		assert: list = self listWithElements reversed;
		should: [list at: 0 put: self absentElement] raise: Error;
		should: [list at: list size + 1 put: self absentElement] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 15:24'!
testBefore
	| list |

	list := self listWithElements.
	self
		assert:
			(self listElements allSatisfy: [:each |
				(list before: each) = (self listElements before: each ifAbsent: [self defaultValue])]).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 15:25'!
testCapacity
	self
		assert: self emptyList capacity >= 0;
		assert: self listWithSingleElement capacity >= 1;
		assert: self listWithElements capacity >= self listElements size! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 15:30'!
testCollect
	self assert:
		(self listWithElements collect: [:each | each asUppercase]) =
			(self listClass newFrom: (self listElements collect: [:each | each asUppercase]))! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:51'!
testCollectFromTo
	| list copySize |

	list := self listWithElements.
	copySize := list size // 2.
	self assert:
		(list collect: [:each | each asUppercase] from: 1 to: copySize) =
			(self listClass newFrom: (self listElements collect: [:each | each asUppercase] from: 1 to: copySize))! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 16:00'!
testCollectThenSelect
	| list |

	list := self listWithElements.
	self assert:
		(list collect: [:each | each asUppercase] thenSelect: [:each | each ~= 'ONE']) =
			(self listWithElements collect: [:each | each asUppercase]) allButFirst! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 9/28/2014 23:40'!
testConcatenation
	self
		assert: (self emptyList, self emptyList) = self emptyList;
		assert: (self listWithElements, self emptyList) = self listWithElements;
		assert: (self emptyList, self listWithElements) = self listWithElements;
		assert:
			(self listWithElements, self listWithElements) =
				(self listClass newFrom: (self listElements, self listElements)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 15:38'!
testCopy
	| list |

	list := self emptyList.
	self assertList: list copy isCopyOf: list.

	list := self listWithSingleElement.
	self assertList: list copy isCopyOf: list.

	list := self listWithElements.
	self assertList: list copy isCopyOf: list.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 9/28/2014 23:33'!
testCopyEmpty
	| list |

	list := self emptyList.
	self assertList: list copyEmpty copiedFrom: list equals: self emptyList.

	list := self listWithSingleElement.
	self assertList: list copyEmpty copiedFrom: list equals: self emptyList.

	list := self listWithElements.
	self assertList: list copyEmpty copiedFrom: list equals: self emptyList.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 9/28/2014 23:33'!
testCopyFromTo
	| list |

	list := self emptyList.
	self
		assertList: (list copyFrom: 1 to: 10)
		copiedFrom: list
		equals: self emptyList.

	list := self listWithElements.
	self
		assertList: (list copyFrom: 1 to: list size)
		copiedFrom: list
		equals: self listWithElements.
	self
		assertList: (list copyFrom: 1 to: (list size / 2) asInteger)
		copiedFrom: list
		equals: (self listClass newFrom: (self listElements copyFrom: 1 to: (list size / 2) asInteger)).
	self
		assertList: (list copyFrom: 1 to: (list size * 2) asInteger)
		copiedFrom: list
		equals: self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 9/28/2014 23:37'!
testCopyReplaceFromToWith
	| list |

	list := self emptyList.
	self
		assertList: (list copyReplaceFrom: 1 to: 1 with: self emptyList)
		copiedFrom: list
		equals: self emptyList.

	list := self listWithElements.
	self
		assertList: (list copyReplaceFrom: 1 to: list size with: self listElements reversed)
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reversed).
	self
		assertList: (list copyReplaceFrom: 0 to: 1 with: self listWithElements)
		copiedFrom: list
		equals: (self listClass newFrom: self listElements, self listElements).
	self
		assertList: (list copyReplaceFrom: list size + 1 to: list size * 2 with: self listWithElements)
		copiedFrom: list
		equals: (self listClass newFrom: self listElements, self listElements).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 15:58'!
testCopyWith
	self assert: (self emptyList copyWith: self singleElement) = self listWithSingleElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 16:02'!
testDefaultValue
	self assert: self emptyList defaultValue = self defaultValue! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:32'!
testDo
	self
		should: [:block | self emptyList do: block] enumerate: #();
		should: [:block | self listWithElements do: block] enumerate: self listElements! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:22'!
testNew
	self assert: (self listClass new: 20) capacity >= 20! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:21'!
testNewFrom
	self assert: (self listClass newFrom: self listElements) = self listWithElements! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:31'!
testNewWithAll
	| list |

	list := self listClass new: 10 withAll: self singleElement.
	self
		assert: list size = 10;
		assert: (list allSatisfy: [:each | each = self singleElement]).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:32'!
testOfSize
	self assert: (self listClass ofSize: 10) size = 10! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 21:16'!
testRemoveAll
	| list |

	list := self listWithElements.
	self
		assert: (list removeAll) == list;
		assert: list isEmpty.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:43'!
testRemoveAt
	| list |

	list := self listWithElements.
	self
		assert: (list removeAt: (list indexOf: self singleElement)) = self singleElement;
		assert: (list = (self listWithElements copyWithout: self singleElement));
		should: [list removeAt: 0] raise: Error;
		should: [list removeAt: list size + 1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:44'!
testRemoveFirst
	| list |

	list := self listWithElements.
	self
		assert: list removeFirst = self listWithElements first;
		assert: list = self listWithElements allButFirst;
		should: [self emptyList removeFirst] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:41'!
testRemoveFirstFew
	| list removedElements |

	list := self listWithElements.
	removedElements := list removeFirst: 2.
	self
		assert: removedElements asArray = (self listElements copyFrom: 1 to: 2);
		assert: list = (self listWithElements allButFirst: 2);
		assert: (list removeFirst: 0) isEmpty;
		should: [list removeFirst: list size + 1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:40'!
testRemoveIfAbsent
	| list |

	list := self listWithElements.
	self
		assert: (list remove: self singleElement ifAbsent: [nil]) = self singleElement;
		deny: (list includes: self singleElement);
		assert: (list remove: self absentElement ifAbsent: [nil]) isNil.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:59'!
testRemoveLast
	| list |

	list := self listWithElements.
	self
		assert: list removeLast = self listWithElements last;
		assert: list = self listWithElements allButLast;
		should: [self emptyList removeLast] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 20:41'!
testRemoveLastFew
	| list removedElements |

	list := self listWithElements.
	removedElements := list removeLast: 2.
	self
		assert: removedElements asArray = (self listElements last: 2);
		assert: list = (self listWithElements allButLast: 2);
		assert: (list removeLast: 0) isEmpty;
		should: [list removeLast: list size + 1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 21:16'!
testReplaceFromToWithStartingAt
	"self debug: #testReplaceFromToWithStartingAt"
	| list |
	list := self listClass new: 5 withAll: 'zzz'.
	self
		assert: (list replaceFrom: 1 to: self listElements size with: self listElements startingAt: 1) == list;
		assert: list = self listWithElements;
		assert: (list replaceFrom: 1 to: self listElements size with: self listElements reversed startingAt: 1) == list;
		assert: list = self listWithElements reversed;
		should: [list replaceFrom: 1 to: self listElements size + 1 with: self listElements startingAt: 1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:08'!
testReverseDo
	self
		should: [:block | self emptyList reverseDo: block] enumerate: #();
		should: [:block | self listWithElements reverseDo: block] enumerate: self listElements reversed.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:12'!
testSelect
	| list |

	list := self listWithElements.
	self assert:
		(list select: [:each | each ~= self singleElement]) =
			((self listClass newFrom: self listWithElements) copyWithout: self singleElement).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:11'!
testSelectThenCollect
	| list |

	list := self listWithElements.
	self assert:
		(list select: [:each | each ~= self singleElement] thenCollect: [:each | each asUppercase]) =
			(self listClass newFrom:
				((self listElements copyWithout: self singleElement) collect: [:each | each asUppercase])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:13'!
testSize
	self
		assert: self emptyList size isZero;
		assert: self listWithSingleElement size = 1;
		assert: self listWithElements size = self listElements size! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:47'!
testSpecies
	self
		assert: self emptyList species = self listSpecies;
		assert: self emptyList speciesForCollect = self listSpecies! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:50'!
testWithCollect
	| interval |

	interval := 1 to: self listElements size.
	self assert:
		(self listWithElements with: interval collect: [:each :i | each -> i]) =
			(self listClass newFrom: (self listElements with: interval collect: [:each :i | each -> i])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:51'!
testWithIndexCollect
	| interval |

	interval := 1 to: self listElements size.
	self assert:
		(self listWithElements withIndexCollect: [:each :i | each -> i]) =
			(self listClass newFrom: (self listElements with: interval collect: [:each :i | each -> i])).! !

TestCase subclass: #XMLParserElementFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLParserElementFactoryTest commentStamp: '<historical>' prior: 0!
This class tests the pluggable element factory. To do this it creates a number of dummy subclasses of XMLElement in its #setUp message that should be removed from the system by its #tearDown message. Since these classes only exist during testing, special accessor messages are used to reference them. These accessors are also generated automatically, whenever you send the class #createDummyElementAccessors. The names for the classes and accessors are taken from the class-side #dummyElementNames message and some supporting messages that specify a prefix and suffix (so the created classes don't collide with other, possibly pre-existing classes in the sytem).

Browse the class side for more information.!

!XMLParserElementFactoryTest class methodsFor: 'utility' stamp: 'JAAyer 11/22/2010 15:18'!
createDummyElementAccessors
	"self createDummyElementAccessors"

	| sourceStream |

	self removeDummyElementAccessors.
	self dummyElementNames
		do: [:each |
			(sourceStream := String new writeStream)
				nextPut: each first asLowercase;
				nextPutAll: each allButFirst, self dummyElementClassSuffix, 'Class';
				cr;
				tab;
				nextPutAll: '^ Smalltalk at: #';
				nextPutAll: (self toDummyElementClassName: each) asString.

			self
				compile: sourceStream contents
				classified: self dummyElementAccessorCategory].! !

!XMLParserElementFactoryTest class methodsFor: 'running' stamp: 'JAAyer 11/20/2010 13:29'!
createDummyElementClasses
	self dummyElementClassNames do: [:each |
		XMLElement
			subclass: each
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: self category]! !

!XMLParserElementFactoryTest class methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 14:02'!
dummyElementAccessorCategory
	^ #'dummy element classes'! !

!XMLParserElementFactoryTest class methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 13:25'!
dummyElementClassNames
	^ self dummyElementNames collect: [:each | self toDummyElementClassName: each]! !

!XMLParserElementFactoryTest class methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 12:52'!
dummyElementClassPrefix
	^ 'XMLTEST'! !

!XMLParserElementFactoryTest class methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 12:52'!
dummyElementClassSuffix
	^ 'Element'! !

!XMLParserElementFactoryTest class methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 15:35'!
dummyElementNames
	"Dummy element class names will be formed from the following base names plus a prefix and suffix.
	
	self createDummyElementAccessors."

	^ #('Default' 'User' 'Username' 'RealName' 'Date' 'Table' 'SerializationFormat' 'Wrong')! !

!XMLParserElementFactoryTest class methodsFor: 'utility' stamp: 'JAAyer 11/20/2010 14:10'!
removeDummyElementAccessors
	(self methodsInCategory: self dummyElementAccessorCategory)
		do: [:each | self removeSelector: each].! !

!XMLParserElementFactoryTest class methodsFor: 'running' stamp: 'JAAyer 11/20/2010 14:10'!
removeDummyElementClasses
	self dummyElementClassNames do: [:each | (Smalltalk at: each) removeFromSystem]! !

!XMLParserElementFactoryTest class methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 14:10'!
toDummyElementClassName: aName
	^ (self dummyElementClassPrefix,
		aName,
		self dummyElementClassSuffix) asSymbol! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
dateElementClass
	^ Smalltalk at: #XMLTESTDateElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
defaultElementClass
	^ Smalltalk at: #XMLTESTDefaultElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
realNameElementClass
	^ Smalltalk at: #XMLTESTRealNameElement! !

!XMLParserElementFactoryTest methodsFor: 'source XML' stamp: 'JAA 3/24/2011 18:26'!
sampleUserXML
	^
	XMLWriter new
		enablePrettyPrinting;
		writeWith: [:writer |
			writer tag
				name: 'user:user';
				xmlnsAt: 'user' put: 'urn:user';
				with: [
					writer
						tag: 'user:username' with: 'jsmith';
						tag: 'user:real-name' with: 'John Smith';
						tag: 'user:date-registered' with: (DateAndTime now - 1 week) printString;
						tag: 'user:date-last-active' with: DateAndTime now printString;
						tag: 'table-name' with: 'users';
						tag: 'row-id' with: (Random new next * 100) asInteger printString;
						tag: 'serializationFormat' with: 'XML']];
		contents! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
serializationFormatElementClass
	^ Smalltalk at: #XMLTESTSerializationFormatElement! !

!XMLParserElementFactoryTest methodsFor: 'running' stamp: 'JAAyer 11/20/2010 13:29'!
setUp
	self class createDummyElementClasses! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
tableElementClass
	^ Smalltalk at: #XMLTESTTableElement! !

!XMLParserElementFactoryTest methodsFor: 'running' stamp: 'JAAyer 11/20/2010 13:29'!
tearDown
	self class removeDummyElementClasses! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 18:25'!
testElementClassAccessor
	| document |

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: XMLPluggableElementFactory new;
			parseDocument.
	self assert: (document allElementsDetect: [:each | (each isMemberOf: XMLElement) not]) isNil.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: (XMLPluggableElementFactory new elementClass: self defaultElementClass);
			parseDocument.
	self assert: (document allElementsDetect: [:each | (each isMemberOf: self defaultElementClass) not]) isNil.! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 03:28'!
testElementClassNameMatching
	| document |

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory:
				(XMLPluggableElementFactory new
					handleElementsMatchingClass: self userElementClass;
					handleElementsMatchingClasses:
						(Array
							with: self realNameElementClass
							with: self serializationFormatElementClass));
			parseDocument.

	self
		assert: ((document root) isMemberOf: self userElementClass);
		assert: ((document root elementAt: 'real-name') isMemberOf: self realNameElementClass);
		assert:
			((document root elementAt: 'serializationFormat')
				isMemberOf: self serializationFormatElementClass).! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 18:08'!
testElementNameAndNamespaceMatching
	| factory document root |

	(factory := XMLPluggableElementFactory new)
		elementClass: self defaultElementClass;
		handleElement: 'user:user' namespaceURI: 'urn:user' withClass: self userElementClass;
		handleElement: 'user:user' withClass: self wrongElementClass;
		handleElement: 'user' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'user' withClass: self wrongElementClass;
		handleElement: 'user:username' withClass: self usernameElementClass;
		handleElement: 'username' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'username' withClass: self wrongElementClass;
		handleElement: 'real-name' namespaceURI: 'urn:user' withClass: self realNameElementClass;
		handleElement: 'real-name' withClass: self wrongElementClass;
		handleElement: 'date-registered' withClass: self dateElementClass;
		handleElement: 'user:date-last-active' namespaceURI: 'urn:notuser' withClass: self wrongElementClass;
		handleElement: 'notuser:date-last-active' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'date-last-active' namespaceURI: 'urn:notuser' withClass: self wrongElementClass;
		handleElement: 'notuser:date-last-active' withClass: self wrongElementClass.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			parseDocument.
	self
		assert: ((root := document root) isMemberOf: self userElementClass);
		assert: ((root elementAt: 'username') isMemberOf: self usernameElementClass);
		assert: ((root elementAt: 'real-name') isMemberOf: self realNameElementClass);
		assert: ((root elementAt: 'date-registered') isMemberOf: self dateElementClass);
		assert:
			((root elements copyAfter: (root elementAt: 'date-registered'))
				allSatisfy: [:each | each isMemberOf: self defaultElementClass]).! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:00'!
testElementsNameAndNamespaceMatching
	| factory document root |

	factory := XMLPluggableElementFactory new.
	factory
		handleElements: #('date-registered' 'date-last-active')
		namespaceURI: 'urn:user'
		withClass: self dateElementClass.
	factory
		handleElements: #('date-registered' 'date-last-active')
		withClass: self wrongElementClass.
	factory
		handleElements: #('table-name' 'not-table-name')
		withClass: self tableElementClass.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			parseDocument.
	root := document root.
	self
		assert: ((root elementAt: 'date-registered') isMemberOf: self dateElementClass);
		assert: ((root elementAt: 'date-last-active') isMemberOf: self dateElementClass);
		assert: ((root elementAt: 'table-name') isMemberOf: self tableElementClass).! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
userElementClass
	^ Smalltalk at: #XMLTESTUserElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
usernameElementClass
	^ Smalltalk at: #XMLTESTUsernameElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
wrongElementClass
	^ Smalltalk at: #XMLTESTWrongElement! !

TestCase subclass: #XMLParserLimitsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLParserLimitsTest methodsFor: 'source' stamp: 'monty 1/19/2014 04:10'!
recursiveEntitiesNamed: aName depth: aLimit isParameter: aBoolean
	| xml |

	xml := String new writeStream.
	1 to: aLimit do: [:i |
		xml
			cr;
			nextPutAll: '<!!ENTITY '.
		aBoolean
			ifTrue: [xml nextPutAll: '% '].
		xml
			nextPutAll: aName, i printString;
			nextPutAll: ' "'.
			(i = aLimit)
				ifTrue: [xml nextPutAll: '<!!--last-->']
				ifFalse: [
					aBoolean
						ifTrue: [xml nextPutAll: '&#37;']
						ifFalse: [xml nextPut: $&].
					xml
						nextPutAll: aName, (i + 1) printString;
						nextPut: $;].
		xml nextPutAll: '">'].
	^ xml contents.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/26/2014 04:29'!
testDefaultLimits
	| parser |

	parser := SAXHandler on: ''.
	self
		assert: parser documentReadLimit = XMLParserLimits defaultDocumentReadLimit;
		assert: parser externalSubsetReadLimit > 0;
		assert: parser maxEntityReplacementDepth > 0.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:19'!
testParserReadLimitTest
	| xml replacement |

	replacement := 'entity replacement'.
	xml := '<!!DOCTYPE test [<!!ENTITY entity "', replacement, '">]><test>&entity;</test>'.
	self
		shouldnt: [
			(SAXHandler on: xml)
				documentReadLimit: xml size + replacement size;
				parseDocument]
		raise: XMLLimitException.
	self
		should: [
			(SAXHandler on: xml)
				documentReadLimit: xml size + replacement size - 1;
				parseDocument]
		raise: XMLLimitException.
	self
		shouldnt: [
			(SAXHandler on: xml)
				documentReadLimit: nil;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 03:54'!
testRecursiveGeneralEntityReplacement
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit
			isParameter: false.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 22:50'!
testRecursiveGeneralEntityReplacementPastLimit
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit + 1
			isParameter: false.
	self
		should: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: nil;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 03:56'!
testRecursiveParameterEntityReplacement
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit
			isParameter: true.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 22:49'!
testRecursiveParameterEntityReplacementPastLimit
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit + 1
			isParameter: true.
	self
		should: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: nil;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/23/2014 17:36'!
testRemovingLimits
	| parser readLimit depthLimit |

	readLimit := 100.
	depthLimit := 5.
	(parser := SAXHandler on: '')
		documentReadLimit: readLimit;
		externalSubsetReadLimit: readLimit;
		maxEntityReplacementDepth: depthLimit.
	self
		assert: parser documentReadLimit = readLimit;
		assert: parser externalSubsetReadLimit = readLimit;
		assert: parser maxEntityReplacementDepth = depthLimit.

	parser removeLimits.
	self
		assert: parser documentReadLimit isNil;
		assert: parser externalSubsetReadLimit isNil;
		assert: parser maxEntityReplacementDepth isNil.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 2/10/2014 02:43'!
testTokenizerMaxPushBack
	| tokenizer |

	tokenizer :=
		XMLWellFormedParserTokenizer
			driver: nil
			on: '' readStream.
	self
		shouldnt: [
			tokenizer streamReader maxNestedStreams
				timesRepeat: [tokenizer streamReader pushBack: 'test']]
		raise: XMLLimitException.
	self
		should: [tokenizer streamReader pushBack: 'test']
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'source' stamp: 'monty 1/19/2014 04:10'!
xmlWithRecursiveEntitiesNamed: aName depth: aLimit isParameter: aBoolean
	| xml |

	(xml := String new writeStream)
		nextPutAll: '<!!DOCTYPE test ['.
	xml nextPutAll:
		(self
			recursiveEntitiesNamed: aName
			depth: aLimit
			isParameter: aBoolean).
	aBoolean
		ifTrue: [xml nextPutAll: '%', aName, '1;]>']
		ifFalse: [xml nextPutAll: ']><test>&', aName, '1;</test>'].

	^ xml contents.! !

TestCase subclass: #XMLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

XMLParserTest subclass: #SAXHandlerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXHandlerTest methodsFor: 'source' stamp: 'monty 1/31/2014 21:05'!
prefixedXMLSource
	^ '<prefix:test xmlns:prefix="prefixNS" xmlns="defaultNS" name="value">test</prefix:test>'! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 12/12/2013 01:04'!
testSAXEntityHandlers
	| xml parser |

	xml := '<!!DOCTYPE root [
		<!!ENTITY % firstDeclaration "<!!ENTITY first &#39; &second; &#39;>">
		%firstDeclaration;
		<!!ENTITY second " replacement ">]>
	<root name="&second;">&first;</root>'.
	(parser :=SAXHandlerProxy saxHandler: (SAXHandler on: xml)) parseDocument.
	self assert:
		#(startDocument
			startDTD:publicID:systemID:
			parameterEntityDeclaration:replacement:
			generalEntityDeclaration:replacement: generalEntityDeclaration:replacement:
			endDTD
			startElement:prefix:uri:localName:attributes:
			startEntity: startEntity: endEntity: endEntity: characters:
			endElement:prefix:uri:localName:
			endDocument) = parser sentHandlerSelectors.
	self assert:
		((parser sentHandlerMessages select: [:each | #(startEntity: endEntity:) includes: each selector])
			collect: [:each | each arguments first]) asArray = #('first' 'second' 'second' 'first').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 2/2/2014 21:57'!
testSAXIgnorableWhitespace
	| xml parser whitespace |

	xml := '
	<one>
		<two>
			test
		</two>
	</one>
	'.
	(parser := SAXHandlerProxy saxHandler: (SAXHandler on: xml)) parseDocument.
	self assert:
		#(startDocument
			startElement:prefix:uri:localName:attributes:
			ignorableWhitespace:
			startElement:prefix:uri:localName:attributes: characters: endElement:prefix:uri:localName:
			ignorableWhitespace:
			endElement:prefix:uri:localName:
			ignorableWhitespace:
			endDocument) = parser sentHandlerSelectors.
	self assert:
		((parser sentHandlerMessages select: [:each | each selector == #ignorableWhitespace:])
			allSatisfy: [:each |
				(whitespace := each arguments first) notEmpty
					and: [whitespace allSatisfy: [:char | char isSeparator]]]).! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 1/31/2014 21:05'!
testSAXPrefixMappingHandlers
	| parser |

	(parser := SAXHandlerProxy saxHandler: (SAXHandler on: self prefixedXMLSource)) parseDocument.
	self assert:
		#(startDocument
			startPrefixMapping:uri: startPrefixMapping:uri:
			startElement:prefix:uri:localName:attributes:
			characters:
			endElement:prefix:uri:localName:
			endPrefixMapping: endPrefixMapping:
			endDocument) = parser sentHandlerSelectors.
	self assert:
		((parser sentHandlerMessages select: [:each |
			#(startPrefixMapping:uri: endPrefixMapping:) includes: each selector])
				collect: [:each | each arguments asArray]) asArray =
					#(#('prefix' 'prefixNS') #('' 'defaultNS') #('prefix') #('')).! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 1/31/2014 21:21'!
testSAXPrefixMappingHandlersNamespacesDisabled
	| parser |

	(parser := SAXHandlerProxy saxHandler: (SAXHandler on: self prefixedXMLSource))
		usesNamespaces: false;
		parseDocument.
	self assert:
		#(startDocument
			startElement:prefix:uri:localName:attributes:
			characters:
			endElement:prefix:uri:localName:
			endDocument) = parser sentHandlerSelectors.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 12/12/2013 01:04'!
testSAXSkippedEntityHandler
	| xml parser |

	xml := '<root>&undeclared;</root>'.
	(parser := SAXHandlerProxy saxHandler: (SAXHandler on: xml))
		isValidating: false;
		parseDocument.
	self assert:
		#(startDocument
			startElement:prefix:uri:localName:attributes:
			skippedEntity:
			endElement:prefix:uri:localName:
			endDocument) = parser sentHandlerSelectors.
	self assert:
		(parser sentHandlerMessages detect: [:each | each selector == #skippedEntity:])
			arguments first = 'undeclared'.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 2/2/2014 21:56'!
testSAXStartElementEndElement
	| parser startElementMessage endElementMessage |

	(parser := SAXHandlerProxy saxHandler: (SAXHandler on: self prefixedXMLSource))
		parseDocument.
	startElementMessage :=
		(parser sentHandlerMessages detect: [:each |
			each selector == #startElement:prefix:uri:localName:attributes:]).
	self assert:
		startElementMessage arguments asArray =
			((Array with: 'prefix:test' with: 'prefix' with: 'prefixNS' with: 'test')
				copyWith: (Dictionary with: 'name' -> 'value')).

	endElementMessage := 
		(parser sentHandlerMessages detect: [:each |
			each selector == #endElement:prefix:uri:localName:]).
	self assert:
		endElementMessage arguments asArray =
			(Array with: 'prefix:test' with: 'prefix' with: 'prefixNS' with: 'test').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 2/2/2014 21:56'!
testSAXStartElementEndElementNamespacesDisabled
	| parser startElementMessage endElementMessage |

	(parser := SAXHandlerProxy saxHandler: (SAXHandler on: self prefixedXMLSource))
		usesNamespaces: false;
		parseDocument.
	startElementMessage :=
		(parser sentHandlerMessages detect: [:each |
			each selector == #startElement:prefix:uri:localName:attributes:]).
	self assert:
		startElementMessage arguments asArray =
			((Array with: 'prefix:test' with: 'prefix' with: '' with: 'test')
				copyWith:
					(Dictionary
						with: 'xmlns:prefix' -> 'prefixNS'
						with: 'xmlns' -> 'defaultNS'
						with: 'name' -> 'value')).

	endElementMessage := 
		(parser sentHandlerMessages detect: [:each |
			each selector == #endElement:prefix:uri:localName:]).
	self assert:
		endElementMessage arguments asArray =
			(Array with: 'prefix:test' with: 'prefix' with: '' with: 'test').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 2/9/2014 02:00'!
testSAXXMLDeclaration
	| parser declarationMessage |

	(parser := SAXHandlerProxy saxHandler:
		(SAXHandler on: '<?xml version="1.0" encoding="UTF-8"?>'))
		parseDocument.
	declarationMessage :=
		(parser sentHandlerMessages detect: [:each |
			each selector == #xmlVersion:encoding:standalone:]).
	self
		assert: declarationMessage arguments first = 1.0;
		assert: declarationMessage arguments second = 'UTF-8';
		deny: declarationMessage arguments third.

	(parser := SAXHandlerProxy saxHandler:
		(SAXHandler on: '<?xml version="1.1" encoding="UTF-16" standalone="yes"?>'))
		parseDocument.
	declarationMessage :=
		(parser sentHandlerMessages detect: [:each |
			each selector == #xmlVersion:encoding:standalone:]).
	self
		assert: declarationMessage arguments first = 1.1;
		assert: declarationMessage arguments second = 'UTF-16';
		assert: declarationMessage arguments third.! !

XMLParserTest subclass: #XMLDOMParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLDOMParserTest class methodsFor: 'source' stamp: 'monty 1/25/2014 04:46'!
addressBookNamespacePrefixedXML
	| xmlWithPrefixes |

	xmlWithPrefixes := self addressBookNamespacedXML.
	#('<addressbook' '</addressbook' '<person' '</person' 'family-name=')
		with: #('<book:addressbook' '</book:addressbook' '<ps:person' '</ps:person' 'ps:family-name=')
		do: [:source :replacement |
			xmlWithPrefixes := xmlWithPrefixes copyReplaceAll: source with: replacement].
	^ xmlWithPrefixes.! !

!XMLDOMParserTest class methodsFor: 'source' stamp: 'monty 1/25/2014 04:46'!
addressBookNamespacedXML
	^ '<addressbook xmlns="defaultNS" xmlns:book="bookNS" xmlns:ps="personNS">',
		(self addressBookXML copyAfter:  $>)! !

!XMLDOMParserTest methodsFor: 'source' stamp: 'monty 1/25/2014 04:44'!
addressBookNamespacePrefixedXML
	^ self class addressBookNamespacePrefixedXML! !

!XMLDOMParserTest methodsFor: 'source' stamp: 'monty 1/25/2014 04:46'!
addressBookNamespacedXML
	^ self class addressBookNamespacedXML! !

!XMLDOMParserTest methodsFor: 'source' stamp: 'monty 12/21/2013 04:00'!
internalSubsetWithDeclarations
	^ '<!!DOCTYPE root PUBLIC "pubid" "external.dtd" [
		<!!ELEMENT root ( #PCDATA | one | two)* >
		<!!ATTLIST root
					name CDATA #IMPLIED
					number ( one | two | three ) "three">
		<!!ENTITY general "replacement">
		<!!ENTITY externalGeneral PUBLIC "pubid2" "external1.dtd">
		<!!ENTITY % parameter "replacement">
		<!!ENTITY % externalParameter SYSTEM "external2.dtd">
		<!!NOTATION notation SYSTEM "note.doc">]>'! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testAllElementsNamespaceURIs
	| document people |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	people :=
		document
			allElementsNamed: 'person'
			namespaceURI: 'defaultNS'.
	self
		assert: people size = 4;
		assert:
			(document
				allElementsNamed: 'person'
				namespaceURI: 'undeclaredNS') isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testAttributeNamespaces
	| document  people |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	people := document allElementsNamed: 'person' namespaceURI: 'personNS'.
	self assert: people notEmpty.
	
	people do: [:each |
		self
			assert: each hasAttributes;
			deny: each attributeNames isEmpty;
			assert:
				(each attributeNames allSatisfy: [:name |
					(each attributeNodeAt: name namespaceURI: 'defaultNS') isNil]);
			assert: (each attributeNodeAt: 'ps:family-name' namespaceURI: 'personNS') notNil].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testAttributeNamespacesDisabled
	| document  people |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacePrefixedXML
			usingNamespaces: false.
	people := document allElementsNamed: 'person'.
	self assert: people notEmpty.
	people do: [:each |
		self
			assert: each hasAttributes;
			deny: each attributeNames isEmpty;
			assert: (each attributeNodeAt: 'ps:family-name' namespaceURI: 'personNS') isNil].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 2/2/2014 22:04'!
testAttributeRenaming
	| doc person |

	doc := XMLDOMParser parse: self addressBookXML.
	person := doc findElementNamed: 'person'.
	(person attributeNodeAt: 'employee-number') name: 'first-name'.
	self
		assert: person attributeNodes size = 2;
		assert: (person attributeAt: 'first-name') = 'A0000';
		assert: (person attributeAt: 'family-name') = 'Gates'.
	self
		should: [(person attributeNodeAt: 'family-name') name: 'undeclared:family-name']
		raise: XMLNamespaceException.
	! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testAttributesExcludingDeclarations
	| document   |

	document := XMLDOMParser parse: self addressBookNamespacedXML.
	self deny: document root hasAttributes.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testAttributesIncludingDeclarations
	| document  root |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacedXML
			usingNamespaces: false.
	root := document root.
	self
		assert: root hasAttributes;
		assert: (root attributeAt: 'xmlns') = 'defaultNS';
		assert: (root attributeAt: 'xmlns:book') = 'bookNS';
		assert: (root attributeAt: 'xmlns:ps') = 'personNS'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:22'!
testCDATASection
	| document element unescaped escaped xml |

	unescaped := 'this is CDATA test content 1 < 2 & 5 > 3'.
	escaped := 'this is CDATA test content 1 &lt; 2 &amp; 5 &gt; 3'.
	xml := 
		'<?xml version="1.0" encoding="UTF-8"?>
		<test-data>test<!![CDATA[', unescaped, ']]>test</test-data>'.
	document := XMLDOMParser parse: xml.
	element := document findElementNamed: 'test-data'.
	self
		assert: element nodes size = 1;
		assert: element firstNode isStringNode;
		assert: element firstNode printString = ('test', escaped, 'test').

	document :=
		(XMLDOMParser on: xml)
			preservesCDataNodes: true;
			parseDocument.
	element := document findElementNamed: 'test-data'.
	self
		assert: element nodes size = 3;
		assert: element nodes second isCData;
		assert: element nodes second printString = ('<!![CDATA[', unescaped, ']]>');
		assert: element strings asArray = (Array with: 'test' with: unescaped with: 'test').! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testChangingNamespaces
	| document people person |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	people := document allElementsNamed: 'person' namespaceURI: 'defaultNS'.
	(person := people third) prefix: 'ps'.
	self assert:
		person == (document findElementNamed: 'person' namespaceURI: 'personNS').
	self assert:
		(people copyWithout: person) =
			((document findElementNamed: 'addressbook') nodesSelect: [:each |
				each isElement and: [each namespaceURI = 'defaultNS']]).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/21/2013 06:10'!
testCollectingParsedNodes
	| document firstPerson allNames |
	"self debug: #testParsing"

	document := XMLDOMParser parse: self addressBookXML.
	firstPerson := document findElementNamed: 'person'.

	allNames := #('person' 'contact-info' 'address' 'job-info' 'manager').
	self
		assert: (firstPerson allElementsCollect: [:each | each name]) asArray = allNames;
		assert: (firstPerson allElements collect: [:each | each name]) asArray = allNames;
		assert: (firstPerson nodesCollect: [:each | each name]) asArray = allNames allButFirst;
		assert: (firstPerson nodes collect: [:each | each name]) asArray = allNames allButFirst.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 00:56'!
testConfiguration
	| document  |

	document := XMLDOMParser parse: self addressBookXML.
	document allElementsDo: [:each |
		self assert: each configuration == document configuration].

	document root innerXML: self addressBookXML.
	document allElementsDo: [:each |
		self assert: each configuration == document configuration].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/11/2014 00:19'!
testDeclaringXMLNamespace
	| xmlNamespace |

	xmlNamespace :=XMLNamespaceScope xmlNamespaceURI.
	self deny:
		((XMLDOMParser parse: self addressBookNamespacedXML) root
			declaresPrefix: 'xml'
			uri: xmlNamespace).

	self assert:
		((XMLDOMParser parse: '<test xmlns:xml="', xmlNamespace, '"/>') root
			declaresPrefix: 'xml'
			uri: xmlNamespace).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/30/2014 21:42'!
testDisabledNamespacesAttributes
	| document |

	document :=
		XMLDOMParser
			parse: '<root xmlns:prefix="prefixNS" xmlns="defaultNS" name="value"/>'
			usingNamespaces: false.
	self
		assert: document root attributeNames asArray = #('xmlns:prefix' 'xmlns' 'name');
		assert: (document root attributeAt: 'xmlns:prefix') = 'prefixNS';
		assert: (document root attributeAt: 'xmlns') = 'defaultNS';
		assert: (document root attributeAt: 'name') = 'value'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/22/2013 06:22'!
testDoctypeDeclaration
	| doctype |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.
	self
		assert: doctype root = 'root';
		assert: doctype publicID = 'pubid';
		assert: doctype systemID = 'external.dtd';
		assert: doctype nodes notEmpty;
		assert: (doctype nodes allSatisfy: [:each | each isDeclaration]).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/27/2013 04:03'!
testDocument
	| document  |

	document := XMLDOMParser parse: self addressBookXML.
	document allElementsDo: [:each | self assert: each document == document].

	document root innerXML: self addressBookXML.
	document allElementsDo: [:each | self assert: each document == document].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/21/2013 06:19'!
testElementNames
	| document root  persons |

	document := XMLDOMParser parse: self addressBookXML.
	root := document root.

	persons := 	root allNodesSelect: [:each |
			each isElement
				and: [(each isNamedAny: #('notperson' 'person'))
					and: [(each isNamed: 'person')
						and: [(each name = 'person') and: [each localName = 'person']]]]].
	self assert: persons size = 4.

	root allElementsNamed: 'person' do: [:p |
		self assert: (persons includes: p).
		persons remove: p].
	self assert: persons isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testElementPrefixes
	| document  persons |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	persons := 	document allElementsSelect: [:each |
		(each isNamedAny: #('notperson' 'ps:person'))
			and: [(each isNamedAny: #('notperson' 'person'))
				and: [(each isNamed: 'person')
					and: [(each isNamed: 'ps:person')
						and: [(each name = 'ps:person') and: [each localName = 'person']]]]]].
	self assert: persons size = 4.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testElementPrefixesNamespacesDisabled
	| document  persons |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacePrefixedXML
			usingNamespaces: false.
	persons := 	document allElementsSelect: [:each |
		(each isNamedAny: #('notperson' 'ps:person'))
			and: [(each isNamedAny: #('notperson' 'person'))
				and: [(each isNamed: 'person')
					and: [(each isNamed: 'ps:person')
						and: [(each name = 'ps:person') and: [each localName = 'person']]]]]].
	self assert: persons size = 4.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 2/2/2014 22:02'!
testElementRenaming
	| document element |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	element := document root.
	element name: 'book:addressbook'.
	self assert: (document elementAt: 'book:addressbook') == element.
	self
		should: [element name: 'undeclared:root']
		raise: XMLNamespaceException.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testFindElementNamespaceURI
	| document people |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	people := document allElementsNamed: 'person' namespaceURI: 'defaultNS'.
	self assert:
		(document
			findElementNamed: 'person'
			namespaceURI: 'defaultNS'
			with: [:each | (each attributeAt: 'employee-number') = 'A7987']) == people last.
	self assert:
		(document
			findElementNamed: 'person'
			namespaceURI: 'defaultNS'
			with: [:each | (each attributeAt: 'family-name') = 'Brown']) == people second.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/12/2014 21:48'!
testInnerXML
	| document contactInfo address1 address2 innerXml |

	document := XMLDOMParser parse: self addressBookXML.
	document allElementsNamed: 'person' do: [:person |
		#('address' 'job-info' 'manager') do: [:each |
			self assert: (person elementAt: each) innerXML isEmpty].

		address1 := (person attributeAt: 'first-name'), '@example.com'.
		address2 := (person attributeAt: 'first-name'), '@users.example.com'.
		innerXml := '<email>', address1, '</email><email2>', address2, '</email2>'.
		contactInfo := person elementAt: 'contact-info'.
		self
			assert: (contactInfo innerXML: innerXml) == contactInfo;
			assert: contactInfo innerXML = innerXml;
			assert: contactInfo elements size = 2;
			assert: (contactInfo elements first isNamed: 'email');
			assert: (contactInfo elements first contentString = address1);
			assert: (contactInfo elements second isNamed: 'email2');
			assert: (contactInfo elements second contentString = address2)].

	self assert: ((document innerXML: '<new-root/>') root isNamed: 'new-root').
	self
		should: [document innerXML: '<root1/><root2/>']
		raise: XMLWellFormednessException.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/31/2013 00:22'!
testInternalAttributeDeclarations
	| doctype firstAttribute secondAttribute |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	firstAttribute := doctype nodes second.
	self
		assert: firstAttribute element = 'root';
		assert: firstAttribute attribute = 'name';
		assert: firstAttribute type = 'CDATA';
		assert: firstAttribute defaultPragma = 'IMPLIED';
		assert: firstAttribute defaultValue isEmpty.

	secondAttribute := doctype nodes third.
	self
		assert: secondAttribute element = 'root';
		assert: secondAttribute attribute = 'number';
		assert: secondAttribute type = '(one |two |three )';
		assert: secondAttribute defaultPragma isEmpty;
		assert: secondAttribute defaultValue = 'three'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/22/2013 06:22'!
testInternalElementDeclaration
	| doctype elementDeclaration |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.
	elementDeclaration := doctype nodes first.
	self
		assert: elementDeclaration element = 'root';
		assert: elementDeclaration contentModel = '(#PCDATA|one|two)*'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/21/2013 06:10'!
testInternalGeneralEntityDeclarations
	| doctype firstEntity secondEntity |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	firstEntity := doctype nodes fourth.
	self
		deny: firstEntity isParameter;
		assert: firstEntity name = 'general';
		assert: firstEntity replacement = 'replacement';
		assert: firstEntity publicID isEmpty;
		assert: firstEntity systemID isEmpty;
		assert: firstEntity ndata isEmpty.

	secondEntity := doctype nodes fifth.
	self
		deny: secondEntity isParameter;
		assert: secondEntity name = 'externalGeneral';
		assert: secondEntity replacement isEmpty;
		assert: secondEntity publicID = 'pubid2';
		assert: secondEntity systemID = 'external1.dtd';
		assert: secondEntity ndata isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/21/2013 06:09'!
testInternalNotationDeclarations
	| doctype notation |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	notation := doctype nodes eighth.
	self
		assert: notation name = 'notation';
		assert: notation publicID isEmpty;
		assert: notation systemID = 'note.doc'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/21/2013 06:09'!
testInternalParameterEntityDeclarations
	| doctype firstEntity secondEntity |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	firstEntity := doctype nodes sixth.
	self
		assert: firstEntity isParameter;
		assert: firstEntity name = 'parameter';
		assert: firstEntity replacement = 'replacement';
		assert: firstEntity publicID isEmpty;
		assert: firstEntity systemID isEmpty.

	secondEntity := doctype nodes seventh.
	self
		assert: secondEntity isParameter;
		assert: secondEntity name = 'externalParameter';
		assert: secondEntity replacement isEmpty;
		assert: secondEntity publicID isEmpty;
		assert: secondEntity systemID = 'external2.dtd'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testNamespaces
	| document  |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	document allElementsDo: [:each |
		(each == document root or: [each isNamed: 'person'])
			ifTrue: [
				self assert: each hasPrefix.
				each == document root
					ifTrue: [
						self
							assert: each prefix = 'book';
							assert: each namespaceURI = 'bookNS']
					ifFalse: [
						self
							assert: each prefix = 'ps';
							assert: each namespaceURI = 'personNS']]
			ifFalse: [
				self
					assert: each namespaceURI = 'defaultNS';
					deny: each hasPrefix;
					assert: each prefix isEmpty]].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesDeclaring
	| document |

	document := XMLDOMParser parse: self addressBookNamespacedXML.
	document allElementsDo: [:each |
		self assert: each hasNamespaces.
		each == document root
			ifTrue: [
				self
					assert: each declaresDefaultNamespace;
					assert: (each declaresPrefix: 'ps' uri: 'personNS')]
			ifFalse: [ 
				self
					deny: each declaresDefaultNamespace;
					deny: (each declaresPrefix: 'ps' uri: 'personNS')].
		self
			deny: (each declaresPrefix: 'undeclared' uri: 'undeclaredNS');
			deny: (each isDeclaredPrefix: 'undeclared');
			deny: (each isDeclaredPrefix: 'undeclared' uri: 'undeclaredNS');
			assert: (each isDeclaredPrefix: 'ps');
			assert: (each isDeclaredPrefix: 'ps' uri: 'personNS')].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesDeclaringDisabled
	| document |

	document := XMLDOMParser parse: self addressBookNamespacedXML usingNamespaces: false.
	document allElementsDo: [:each |
		self
			deny: each hasNamespaces;
			deny: each declaresDefaultNamespace;
			deny: (each declaresPrefix: 'ps' uri: 'personNS');
			deny: (each isDeclaredPrefix: 'ps');
			deny: (each isDeclaredPrefix: 'ps' uri: 'personNS')].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testNamespacesDisabled
	| document |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacePrefixedXML
			usingNamespaces: false.
	document allElementsDo: [:each |
		self assert: each namespaceURI isEmpty.
		(each == document root or: [each isNamed: 'person'])
			ifTrue: [
				self
					assert: each hasPrefix;
					assert:
						each prefix =
							((each == document root) ifTrue: ['book'] ifFalse: ['ps'])]
			ifFalse: [
				self
					deny: each hasPrefix;
					assert: each prefix isEmpty]].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesResolving
	| document |

	document := XMLDOMParser parse: self addressBookNamespacedXML.
	document allElementsDo: [:each |
		self
			assert: each defaultNamespace = 'defaultNS';
			assert: (each resolvePrefix: '') = 'defaultNS';
			assert: (each resolvePrefix: 'book') = 'bookNS';
			assert: (each resolvePrefix: 'ps') = 'personNS';
			assert:
				each namespaceURI =
					(each hasPrefix
						ifTrue: [each resolvePrefix: each prefix]
						ifFalse: [each defaultNamespace])]! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesResolvingDisabled
	| document |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacedXML
			usingNamespaces: false.
	document allElementsDo: [:each |
		self
			assert: each defaultNamespace isEmpty;
			assert: (each resolvePrefix: '') isEmpty;
			assert: (each resolvePrefix: 'book') isEmpty;
			assert: (each resolvePrefix: 'ps') isEmpty;
			assert:
				each namespaceURI =
					(each hasPrefix
						ifTrue: [each resolvePrefix: each prefix]
						ifFalse: [each defaultNamespace])]! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/21/2013 06:21'!
testNodeDetecting
	| document people |

	document := XMLDOMParser parse: self addressBookXML.
	people := document root elementsAt: 'person'.
	self assert:
		((document findElementNamed: 'addressbook') nodesDetect: [:each |
			each isElement and: [(each attributeAt: 'family-name') = 'Brown']]) == people second.
	self assert:
		((document findElementNamed: 'addressbook')
			nodesDetect: [:each | each isElement and: [(each attributeAt: 'unknown') = 'Brown']]
			ifNone: ['not found']) = 'not found'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/12/2013 01:06'!
testNodeListClass
	| document parser listClass |

	listClass := XMLCachingNodeList.
	document :=
		(parser := XMLDOMParser on: self addressBookXML)
			nodeListClass: listClass;
			parseDocument.

	self
		assert: parser nodeListClass == listClass;
		assert: document nodes class == listClass;
		assert: (document allElements allSatisfy: [:each | each nodes class == listClass]).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/13/2014 01:37'!
testOverridingDefaultNamespace
	| document |

	document := XMLDOMParser parse: '<outer xmlns="default"><inner xmlns=""/></outer>'.
	self
		assert: document root namespaceURI = 'default';
		assert: document root defaultNamespace = 'default';
		assert: document root firstNode namespaceURI isEmpty;
		assert: document root firstNode defaultNamespace isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/31/2014 22:22'!
testParseDocumentWhile
	| i document |

	self
		shouldnt: [
			i := 0.
			document :=
				(XMLDOMParser on: '<root><one>two<#malformed')
					parseDocumentWhile: [(i := i + 1) <= 4]]
		raise: XMLWellFormednessException.

	self
		assert: document allNodes size = 4;
		assert: (document findElementNamed: 'one') nodes size = 1;
		assert: (document findElementNamed: 'one') contentString = 'two'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/12/2013 00:41'!
testParsingAttributes
	| document firstPerson  |

	document := XMLDOMParser parse: self addressBookXML.
	firstPerson := document findElementNamed: 'person'.
	self
		assert: (firstPerson attributeAt: 'employee-number') = 'A0000';
		assert: (firstPerson attributeAt: 'family-name') = 'Gates';
		assert: (firstPerson attributeAt: 'first-name') = 'Bob';
		assert:
			firstPerson attributeNames asArray =
				#('employee-number' 'family-name' 'first-name');
		assert: firstPerson attributeAssociations asArray =
			{('employee-number'->'A0000'). ('family-name'->'Gates'). ('first-name'->'Bob').}.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/12/2013 00:42'!
testParsingRoot
	| document root  |

	document := XMLDOMParser parse: self addressBookXML.
	self assert: (document isKindOf: XMLDocument).
	root := document root.
	self assert: (root class == XMLElement).

	self assert:
		(root allNodesDetect: [:each |
			each isElement
				and: [each isNamedAny: #('AddressBook' 'addressbook')]]) == root.
	self assert: (root findElementNamed: 'addressbook') == root.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/12/2013 01:09'!
testXMLComment
	| document comment|

	document := XMLDOMParser parse: self addressBookXML.
	self deny: (document findElementNamed: 'contact-info') hasChildren.

	document :=
		(XMLDOMParser on: self addressBookXML)
			preservesCommentNodes: true;
			parseDocument.
	comment := (document findElementNamed: 'contact-info') firstNode.
	self
		assert: comment notNil;
		assert: comment string = 'Confidential'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 2/9/2014 00:22'!
testXMLDeclaration
	| document |

	document := XMLDOMParser parse: '<?xml version="1.0" encoding="UTF-8"?>'.
	self
		assert: document version = 1.0;
		assert: document encoding = 'UTF-8';
		deny: document isStandalone.

	document := XMLDOMParser parse: '<?xml version="1.1" encoding="UTF-16" standalone="yes"?>'.
	self
		assert: document version = 1.1;
		assert: document encoding = 'UTF-16';
		assert: document isStandalone.! !

!XMLParserTest class methodsFor: 'source' stamp: 'monty 1/12/2014 23:36'!
addressBookXML
	^'<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest class methodsFor: 'source' stamp: 'monty 1/12/2014 23:36'!
addressBookXMLWithDTD
	^'<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE addressbook SYSTEM "addressbook.dtd">
<?xml-stylesheet type="text/xsl" href="demo.xsl"?>
<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest methodsFor: 'source' stamp: 'monty 1/12/2014 23:36'!
addressBookXML
	^ self class addressBookXML! !

!XMLParserTest methodsFor: 'source' stamp: 'monty 1/12/2014 23:37'!
addressBookXMLWithDTD
	^ self class addressBookXMLWithDTD! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAA 3/24/2011 18:26'!
benchmark1
	"
	XMLParserTest new benchmark1
	600 timesRepeat => 2131 (Alexandre Bergel 3/29/2010 09:09)
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [SAXHandler parse: self addressBookXML]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAA 3/24/2011 18:26'!
benchmark2
	"
	XMLParserTest new benchmark2
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [XMLDOMParser parse: self addressBookXML]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'monty 2/6/2014 19:17'!
benchmark3
	"
	XMLParserTest new benchmark3
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [XMLDOMParser parse: self addressBookXMLWithDTD]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'monty 11/16/2013 07:34'!
benchmark4
	| doc |
	"
	XMLParserTest new benchmark4
	"
	doc := XMLDOMParser
		parse: self addressBookXML readStream
		usingNamespaces: false.
	^ [500000 timesRepeat: [
		doc root firstElement
			elementAt: 'contact-info';
			elementAt: 'address';
			elementAt: 'job-info';
			elementAt: 'manager']] timeToRun.! !

!XMLParserTest methodsFor: 'source' stamp: 'monty 12/19/2013 02:04'!
elementDeclarationsNamed: aNameCollection
	| stream |

	stream := String new writeStream.
	aNameCollection do: [:each |
		stream
			nextPutAll: '<!!ELEMENT ';
			nextPutAll: each;
			nextPutAll: ' EMPTY>';
			cr].
	^ stream contents.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/6/2014 03:58'!
testAttributeDefaultDeclaration
	| dtd document |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root one CDATA #IMPLIED
						two CDATA #REQUIRED
						three CDATA #FIXED "fixedDefault"
						three CDATA #FIXED "redundant"
						four CDATA "default" >]>'.
	document := XMLDOMParser parse: dtd, '<root two=''required'' three="nonDefault" />'.
	self
		assert: (document root attributeAt: 'two') = 'required';
		deny: (document root includesAttribute: 'one');
		assert: (document root attributeAt: 'three') = 'fixedDefault';
		assert: (document root attributeAt: 'four') = 'default'.
	self should: [XMLDOMParser parse: dtd, '<root />'] raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 19:17'!
testAttributeEntityDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ENTITY firstEntity SYSTEM "first.txt" NDATA first>
		<!!ENTITY secondEntity SYSTEM "second.txt" NDATA second>
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root
			unparsedEntity ENTITY #IMPLIED
			unparsedEntities ENTITIES #IMPLIED>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root unparsedEntity="firstEntity"  unparsedEntities="firstEntity  secondEntity"/> ']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root unparsedEntity="thirdEntity"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root unparsedEntity="secondEntity thirdEntity"/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 19:18'!
testAttributeEnumerationDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!NOTATION txtFile PUBLIC "//file.txt//">
		<!!NOTATION confFile PUBLIC "//file.conf//">
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root
			number ( one | two | three) #IMPLIED
			note NOTATION (txtFile | confFile ) #IMPLIED>]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<root number="two" note="txtFile"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root number="four"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root note="htmlFile" />']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 19:19'!
testAttributeIDDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root (unique|unique2)+>
		<!!ELEMENT unique EMPTY>
		<!!ELEMENT unique2 EMPTY>
		<!!ATTLIST unique id ID #REQUIRED>
		<!!ATTLIST unique2 id ID #REQUIRED>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root><unique id=" one "/><unique id="two"/><unique2 id="three"/></root> ']
		raise:XMLValidationException.
	self
		should: [
			SAXHandler parse:
				dtd, '<root><unique id="one"/><unique id=" one "/></root> ']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ATTLIST unique
						id ID #REQUIRED
						id2 ID #REQUIRED>]>
					<root/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 19:54'!
testAttributeIDRefDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root (unique|ref|refs)+>
		<!!ELEMENT unique EMPTY>
		<!!ELEMENT ref EMPTY>
		<!!ELEMENT refs EMPTY>
		<!!ATTLIST unique id ID #REQUIRED>
		<!!ATTLIST ref idref IDREF #REQUIRED>
		<!!ATTLIST refs idrefs IDREFS #REQUIRED>]>'.
	XMLDOMParser parse:
		dtd, '<root>
		<unique id=" one "/><unique id=" two "/>
		<ref idref="one"/><refs idrefs="  one  two"/>
		</root> '.
	self
		should: [
			XMLDOMParser parse:
				dtd, '<root><unique id="one"/><ref idref="two"/></root> ']
		raise: XMLValidationException.
	self
		should: [
			XMLDOMParser parse:
				dtd, '<root><unique id="one"/><refs idrefs="one two"/></root> ']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 00:58'!
testAttributeNmtokenDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root ANY>
		<!!ATTLIST root name NMTOKEN #IMPLIED>
		<!!ATTLIST root names NMTOKENS #IMPLIED>]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<root name=" one " names="  one  two three "/> ']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root name="#" names="one two"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root name="one" names="# #" />']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 11/19/2013 05:55'!
testAttributeValueNormalization
	| separators unnormalized normalized document |

	separators :=
		String
			with: Character space
			with: Character tab
			with: Character lf.
	unnormalized := separators, 'nonspace', separators.
	normalized := '   nonspace   '.
	document :=
		(XMLDOMParser parse:
			'<root name="', unnormalized,'">', unnormalized, '</root>').
	self
		assert: (document root attributeAt: 'name') = normalized;
		assert: (document root contentString) = unnormalized.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 10/9/2014 00:55'!
testDeclaringNamespaces
	self
		shouldnt: [SAXHandler parse: '<prefix:root prefix:name="test" xmlns:prefix="NS"/>']
		raise: XMLNamespaceException.
	self
		shouldnt: [SAXHandler parse: '<root xmlns="defaultNS"/>']
		raise: XMLNamespaceException.
	self
		shouldnt: [SAXHandler parse: '<root xmlns=""/>' usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 10/9/2014 20:40'!
testDeclaringNamespacesBadPrefixes
	self
		should: [SAXHandler parse: '<root xmlns:invalid:prefix="invalidNS"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root xmlns::invalidprefix="invalidNS"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root xmlns:invalidprefix:="invalidNS"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<:invalidname />']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<invalidname: />']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root :invalidname="test" />']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root invalidname:="test" />']
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 10/9/2014 00:58'!
testDeclaringNamespacesBadURIs
	self
		should: [SAXHandler parse: '<root xmlns:empty=""/>' usingNamespaces: true]
		raise: XMLNamespaceException.

	"Cannot change xmlns or xml prefixes"
	self
		should: [SAXHandler parse: '<root xmlns:xmlns="newNS"/>' usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root xmlns:xml="newNS" />' usingNamespaces: true]
		raise: XMLNamespaceException.

	"Redundant xml prefix mapping is ok, but xmlns is not"
	self
		shouldnt: [
			SAXHandler
				parse: '<root xmlns:xml="', XMLNamespaceScope xmlNamespaceURI, '"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse: '<root xmlns:xmlns="', XMLNamespaceScope xmlnsNamespaceURI, '"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 18:55'!
testDefaultDocumentURI
	| cwd documentUri |

	documentUri := (SAXHandler on: '' documentURI: 'test/test.xml') documentURI.
	self assert: documentUri = 'test/test.xml'.

	cwd := XMLFileHandle currentWorkingDirectory.
	documentUri := (SAXHandler on: '') documentURI.
	self
		assert: documentUri size > (cwd size + 1);
		assert: (documentUri copyUpToLast: $/) = cwd.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 11/26/2013 21:42'!
testDoctypeRoot
	| xml |

	xml := '<!!DOCTYPE root><invalidRoot/>'.
	self
		should: [SAXHandler parse: xml]
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler
				parse: xml
				usingNamespaces: false
				validation: false]
		raise: XMLValidationException ! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 18:39'!
testElementAnyAndEmptyDeclarations
	self
		shouldnt: [
			SAXHandler parse: 
				'<!!DOCTYPE root [
					<!!ELEMENT root ANY>
					<!!ELEMENT two ANY>]>
				<root>one<two>three</two>four</root>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse: 
				'<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root> <!!--empty--> </root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse: 
				'<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root><non-empty/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/27/2013 07:10'!
testElementContentModelsBranching
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root ((one|two),(three|four|five))>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><three/></root>';
				parse: dtd, '<root><one/><four/></root>';
				parse: dtd, '<root><one/><five/></root>';
				parse: dtd, '<root><two/><three/></root>';
				parse: dtd, '<root><two/><four/></root>';
				parse: dtd, '<root><two/><five/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><three/><four/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><four/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 01:01'!
testElementContentModelsManyBranches
	| dtd names |

	names := (1 to: 25) collect: [:each | 'element', each printString].
	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: names),
		'<!!ELEMENT root (', ($| join: names), ')>]>'.

	names do: [:each |
		self
			shouldnt: [SAXHandler parse: dtd, '<root><', each, '/></root>']
			raise: XMLValidationException].
	self
		should: [
			SAXHandler parse:
				dtd, '<root><element', (names size + 1) printString, '/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/27/2013 16:12'!
testElementContentModelsOptional
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root (one?,two,three?,(four?, five)?)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><three/></root>';
				parse: dtd, '<root><one/><two/><three/><five/></root>';
				parse: dtd, '<root><two/></root>';
				parse: dtd, '<root><one/><two/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><three/><four/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/><four/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/><five/><four/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/28/2013 02:01'!
testElementContentModelsOptionalBranching
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five' 'six')),
		'<!!ELEMENT root (((one,two)|(three,four)),five?)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><five/></root>';
				parse: dtd, '<root><three/><four/><five/></root>';
				parse: dtd, '<root><one/><two/></root>';
				parse: dtd, '<root><three/><four/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><three/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><five/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/28/2013 01:36'!
testElementContentModelsOptionalEmpty
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three')),
		'<!!ELEMENT root (one?,two)?>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/></root>';
				parse: dtd, '<root><two/></root>';
				parse: dtd, '<root></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><one/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><three/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><one/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/27/2013 08:40'!
testElementContentModelsPlus
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root ((one?,two)+,three,four+,five)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><one/><two/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><three/><four/><four/><five/></root>';
				parse: dtd, '<root><two/><three/><four/><five/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><three/><four/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><four/><four/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/><five/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/27/2013 16:08'!
testElementContentModelsStar
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root ((one?,two)*,(three)*,four,five*)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><three/><three/><four/><five/></root>';
				parse: dtd, '<root><four/><five/><five/></root>';
				parse: dtd, '<root><four/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><three/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/28/2013 01:49'!
testElementContentModelsStarEmpty
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three')),
		'<!!ELEMENT root (one*,two)*>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><one/><two/><two/></root>';
				parse: dtd, '<root><two/><one/><two/></root>';
				parse: dtd, '<root><two/><two/></root>';
				parse: dtd, '<root></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><one/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><three/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 18:30'!
testElementMixedContentDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root (#PCDATA | one | two | three)*>
		<!!ELEMENT one EMPTY>
		<!!ELEMENT two (#PCDATA)>
		<!!ELEMENT three (#PCDATA)*>
		<!!ELEMENT four EMPTY>]>'.
	self
		shouldnt: [
			XMLDOMParser parse:
				dtd, '<root>text<one/><two><!![CDATA[text]]></two></root>']
		raise: XMLValidationException.
	self
		shouldnt: [XMLDOMParser parse: dtd, '<root><three>text<!!--comment--></three></root>']
		raise: XMLValidationException.
	self
		should: [XMLDOMParser parse: dtd, '<root><four/></root>']
		raise: XMLValidationException.
	self
		should: [XMLDOMParser parse: dtd, '<root><two><three/></two></root>']
		raise: XMLValidationException.
	self
		should: [
			XMLDOMParser parse: "missing *"
				'<!!DOCTYPE root [<!!ELEMENT root (#PCDATA|one)>]><root/>']
		raise: XMLValidationException.
	'+*?,' do: [:each |
		self
			should: [
				XMLDOMParser parse:
					'<!!DOCTYPE root [<!!ELEMENT two (#PCDATA', each asString, ')>]><root/>']
			raise: XMLValidationException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/13/2014 00:16'!
testElementWellFormedness
	| error |

	error := XMLWellFormednessException.
	self
		should: [SAXHandler parse: '<root1>'] raise: error;
		should: [SAXHandler parse: '</root1>'] raise: error;
		should: [SAXHandler parse: '<root1/><root2/>'] raise: error;
		should: [SAXHandler parse: '<root></unopened></root>'] raise: error;
		should: [SAXHandler parse: '<root><unclosed></root>'] raise: error;
		should: [SAXHandler parse: '<root><first><second></first></second></root>'] raise: error.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/8/2013 01:54'!
testEntityReplacement
	| document |

	document := XMLDOMParser parse: '
		<!!DOCTYPE root [
			<!!ENTITY % partialDeclaration "name1 &#39;replacement1&#x27;">
			<!!ENTITY % name1Declaration "<!!ENTITY %partialDeclaration;>">
			%name1Declaration;
			<!!ENTITY name2 "replacement2 &name1;" >
			<!!ENTITY name3 "replacement3 &name2;" >
			<!!ENTITY quotes "&#x27;quoted &#x22;value&#34;&#39;">
		]>
		<root name="&quotes;">&name1; &name2; &name3; ',
		'%name1Declaration;</root>'.
	self assert:
		document root contentString =
			('replacement1 ',
			'replacement2 replacement1 ',
			'replacement3 replacement2 replacement1 ',
			'%name1Declaration;').
	self assert:
		(document root attributeAt: 'name') =
			(('quoted "value"' copyWithFirst: $') copyWith: $').! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/28/2014 05:18'!
testEntityURIInheritance
	| xml parser driver |

	xml := '<!!DOCTYPE root [%first;]><root xml:base="/new/xml">&second;</root>'.
	parser :=
		(SAXHandler
			on: xml
			documentURI: 'test.xml') parser.
	(driver := parser driver)
		dtd parameterEntities
			at: 'first'
			put:
				(DTDExternalParameterEntity
					name: 'first'
					replacement: '<!!ENTITY second "<!!--comment-->">'
					uri: 'test.ent').

	self assert: driver currentURI = 'test.xml'.

	3 timesRepeat: [parser parseToken].
	self assert: driver currentURI = 'test.ent'.

	2 timesRepeat: [parser parseToken].
	self assert: driver currentURI = 'test.xml'.

	2 timesRepeat: [parser parseToken].
	self assert: driver currentURI = '/new/xml'.

	3 timesRepeat: [parser parseToken].
	self
		assert: driver currentURI = 'test.xml';
		assert: parser atEnd.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 23:27'!
testIncompleteCDataTokens
	#('<test><!![' '<test><!![CDATA[' '<test><!![CDATA[ test'
		'<test><!![CDATA[ test ]' '<test><!![CDATA[ test ]]')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 23:20'!
testIncompleteCommentTokens
	#('<!!--' '<!!-- comment' '<!!-- comment -' '<!!-- comment --')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/23/2014 03:12'!
testIncompleteDoctypeTokens
	#('<!!DOCTYPE' '<!!DOCTYPE >'
		'<!!DOCTYPE test' '<!!DOCTYPE test PUBLIC' '<!!DOCTYPE test PUBLIC "pubid'
		'<!!DOCTYPE test PUBLIC "pubid" "sysid' '<!!DOCTYPE test PUBLIC "pubid" "sysid"'
		'<!!DOCTYPE test SYSTEM' '<!!DOCTYPE test SYSTEM "sysid' '<!!DOCTYPE test SYSTEM "sysid"'
		'<!!DOCTYPE test [' '<!!DOCTYPE test []')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 23:22'!
testIncompleteEndTagTokens
	#('<test></' '<test></test')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 23:23'!
testIncompletePITokens
	#('<?name' '<?name value' '<?name value?' '<?name value>')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 23:19'!
testIncompleteStartTagTokens
	#('<' '<test' '<test name'
		'<test name=' '<test name="value' '<test name="value"'
		'<test name="value" /')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/22/2014 23:17'!
testIncompleteXMLDeclarationTokens
	#('<?xml' '<?xml ?' '<?xml ?>'
		'<?xml version' '<?xml version=' '<?xml version="1.0'
		'<?xml version="1.0" encoding' '<?xml version="1.0" encoding="UTF-8'
		'<?xml version="1.0" encoding="UTF-8" standalone'
		'<?xml version="1.0" encoding="UTF-8" standalone="yes'
		'<?xml version="1.0" encoding="UTF-8" standalone="yes"'
		'<?xml version="1.0" encoding="UTF-8" standalone="yes"?')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/6/2013 20:35'!
testInvalidElementContentModels
	self
		should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root ()>]>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root (one,)>]>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root (one|)>]>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root (one)(two)>]>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root (one|,two)>]>']
		raise: XMLValidationException.
	'|,?*+' do: [:each |
		self
			should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root (', each asString, 'one)>]>']
			raise: XMLValidationException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/7/2013 00:15'!
testLineEndingsDoNotMatter
	| source docFromSourceWithCrs docFromSourceWithLfs docFromSourceWithCrlfs |

	source := '<root>{1}one{1}two{1}</root>'.
	docFromSourceWithCrs := XMLDOMParser parse: (source format: (Array with: String cr)).
	docFromSourceWithLfs := XMLDOMParser parse: (source format: (Array with: String lf)).
	docFromSourceWithCrlfs := XMLDOMParser parse: (source format: (Array with: String crlf)).
	self
		assert: (docFromSourceWithCrs root contentString) = (docFromSourceWithLfs root contentString);
		assert: (docFromSourceWithLfs root contentString) = (docFromSourceWithCrlfs root contentString).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/31/2014 00:15'!
testNamespacePrefixAliasing
	| declarations |

	declarations := '<test xmlns:alias1="ns" xmlns:alias2="ns" '.
	self
		shouldnt: [
			SAXHandler
				parse: declarations, ' alias1:name="value" />'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse: declarations, ' alias1:name="value" alias2:name="value"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		shouldnt: [
			SAXHandler
				parse: declarations, ' alias1:name="value" alias2:name2="value"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/31/2014 00:24'!
testNamespacePrefixInheritance
	self
		shouldnt: [
			SAXHandler
				parse: '<one xmlns:prefix1="ns"><prefix1:two /></one>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse:
					'<one xmlns:prefix1="ns">
						<prefix2:two><three xmlns:prefix2="ns2"/></prefix2:two>
					</one>'
				usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'StephaneDucasse 12/27/2013 16:52'!
testParserLittleDocument

	| doc |
	doc := XMLDOMParser parse: 
'<?xml version="1.0" encoding="UTF-8"?>
<!!--an example-->
<example:user xmlns:example="urn://example">
    <username>jsmith</username>
    <real-name>John Smith</real-name>
    <age>25</age>
    <id>101</id>
</example:user>' readStream.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/10/2014 02:43'!
testPredefinedEntityReplacement
	| document references replacements |

	references := String new writeStream.
	replacements := String new writeStream.
	XMLWellFormedParserTokenizer predefinedEntities keysAndValuesDo: [:name :char |
		references
			nextPut: $&;
			nextPutAll: name;
			nextPut: $;.
		replacements nextPut: char].

	document :=
		XMLDOMParser parse:
			('<!!DOCTYPE root [
				<!!ENTITY nonRecursive "&amp;nonRecursive;">]>
			<root name=''{1}''>
				<one>{1}</one>
				<two>&nonRecursive;</two>
			</root>' format: (Array with: references contents)).
	self
		assert: (document root attributeAt: 'name') = replacements contents;
		assert: (document findElementNamed: 'one') contentString = replacements contents;
		assert: (document findElementNamed: 'two') contentString = '&nonRecursive;'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 23:49'!
testPrefixedAttributeDeclarations
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root
			prefix:name CDATA #IMPLIED
			unprefixed CDATA #IMPLIED>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root xmlns:prefix="ns" prefix:name="value" unprefixed="value"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root name="value" unprefixed="value" />']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				dtd, '<root xmlns:prefix="ns" prefix:name="value" prefix:unprefixed="value"/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 23:44'!
testPrefixedElementDeclarations
	| dtd |

	dtd := '<!!DOCTYPE prefix:root [
		<!!ELEMENT prefix:root ANY>
		<!!ELEMENT unprefixed EMPTY>]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<prefix:root xmlns:prefix="ns"><unprefixed/></prefix:root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><unprefixed/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<prefix:root xmlns:prefix="ns"><prefix:unprefixed/></prefix:root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/7/2013 20:35'!
testPreservesUndeclaredEntityReferences
	| xml parser document |

	xml := '<!!DOCTYPE root [
		<!!ENTITY unreferenced "%undeclaredOne;">
		%undeclaredOne;]>
		<root name="&undeclaredTwo;">&undeclaredThree;</root>'.
	(parser := XMLDOMParser on: xml)
		isValidating: false;
		preservesUndeclaredEntityReferences: true.
	document := parser parseDocument.

	self
		assert: document doctypeDeclaration nodes size = 1;
		assert: document doctypeDeclaration firstNode replacement = '%undeclaredOne;';
		assert: (document root attributeAt: 'name') = '&undeclaredTwo;';
		assert: (document root contentString) = '&undeclaredThree;'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/15/2014 00:51'!
testProcessingInstructions
	| document |

	document :=
		XMLDOMParser parse:
			'<?xml version="1.1" standalone="yes"?><?firstTarget first>Data?>',
			'<root><?secondTarget second?Data?></root> '.

	self
		assert: document version = 1.1;
		assert: document nodes size = 2;
		assert: document firstNode isPI;
		assert: document firstNode target = 'firstTarget';
		assert: document firstNode data = 'first>Data';
		assert: document root firstNode target = 'secondTarget';
		assert: document root firstNode data = 'second?Data'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/21/2013 20:07'!
testRequiredDTDDeclarations
	self
		shouldnt: [
			(SAXHandler on: '<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root/>')
				requiresDocumentConstraints: true;
				parseDocument]
		raise: XMLValidationException.
	self
		should: [
			(SAXHandler on: '<root/>')
				requiresDocumentConstraints: true;
				parseDocument]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/4/2014 07:44'!
testSelfReferentialEntityReplacement
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ENTITY first "&first;">]><root>&first;</root>']
		raise: XMLWellFormednessException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ENTITY % first "&#37;first;">%first;]><root/>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/13/2014 22:14'!
testStandalone
	| dtdXml standaloneDecl |

	dtdXml := self addressBookXMLWithDTD.
	self
		shouldnt: [
			SAXHandler
				parse: dtdXml
				usingNamespaces: true
				validation: true
				externalEntities: false]
		raise: XMLFileException.

	standaloneDecl := (dtdXml copyUpTo: $>) allButLast, ' standalone="yes" ?>'.
	self
		shouldnt: [SAXHandler parse: standaloneDecl, (dtdXml copyAfter: $>)]
		raise: XMLFileException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/12/2014 23:20'!
testSubsetEndingEntityReplacement
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY % endOfSubset "]>">
					%endOfSubset;
					<root/>']
		raise: XMLWellFormednessException! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 21:53'!
testUndeclaredAttributes
	self
		shouldnt: [
			SAXHandler
				parse: '<!!DOCTYPE root [<!!ENTITY ignored "ignored">]><root name="value"/>'
				usingNamespaces:true
				validation: true]
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ELEMENT root ANY>
					<!!ATTLIST root name CDATA #IMPLIED>]><root name="value"></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ELEMENT root ANY>]><root name="value"></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 20:04'!
testUndeclaredElements
	self
		shouldnt: [
			SAXHandler
				parse: '<!!DOCTYPE root [<!!ENTITY ignored "">]><root/>'
				usingNamespaces:true
				validation: true]
		raise:XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ELEMENT root ANY>]><root><undeclared/></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ATTLIST root name CDATA #IMPLIED>]><root/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/6/2013 19:04'!
testUndeclaredEntities
	| document dtd content |

	dtd := '<!!DOCTYPE root [<!!ENTITY declared "%undeclared;">]>'.
	content := '<root>&undeclared;test&undeclared;</root>'.
	self should: [XMLDOMParser parse: content] raise: XMLValidationException.
	self should: [XMLDOMParser parse: dtd, '<root/>'] raise: XMLValidationException.
	self
		shouldnt: [
			document :=
				XMLDOMParser
					parse: dtd, content
					usingNamespaces: false
					validation: false]
		raise: XMLValidationException.
	self assert: document allNodes second contentString = 'test'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/6/2013 17:39'!
testUndeclaredNamespaces
	self
		shouldnt: [SAXHandler parse: '<prefix:root prefix:name="test"/>' usingNamespaces: false]
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<prefix:root/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root prefix:name="test"/>']
		raise: XMLNamespaceException.

	"xml is OK as a prefix, xmlns is not"
	self
		shouldnt: [SAXHandler parse: '<xml:root xml:name="test"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<xmlns:root />']
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 11/20/2013 01:06'!
testUnparsedEntities
	| document |

	document := XMLDOMParser parse: '
		<!!DOCTYPE root [
			<!!NOTATION note PUBLIC "-//ext//note//EN">
			<!!ENTITY unparsedEntity SYSTEM "note.txt" NDATA note>
		]>
		<root notation=&unparsedEntity; />'.
	self assert: (document root attributeAt: 'notation') = 'note'.
	self
		should: [XMLDOMParser parse: '<root attr=&undeclaredUnparsedEntity;/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 02:49'!
testValidationAndConstraintsToggle
	self assert:
		((SAXHandler on: '')
			isValidating: false;
			requiresDocumentConstraints: true;
			isValidating).
	self deny:
		((SAXHandler on: '')
			requiresDocumentConstraints: true;
			isValidating: false;
			requiresDocumentConstraints).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 03:24'!
testValidationAndEntityReferenceToggle
	self deny:
		((SAXHandler on: '')
			isValidating: true;
			preservesUndeclaredEntityReferences: true;
			isValidating).
	self deny:
		((SAXHandler on: '')
			preservesUndeclaredEntityReferences: true;
			isValidating: true;
			preservesUndeclaredEntityReferences).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/23/2014 03:07'!
testWhitespacePreservation
	| xml root notPreserved preserved |

	xml :=
		'<root xml:space="preserve">
			<not-preserved xml:space="default">
				<preserved xml:space="preserve">
				</preserved>
			</not-preserved>
		</root>'.
	root := (XMLDOMParser parse: xml) root.
	self
		assert: root nodes size = 3;
		assert: root stringNodes size = 2.
	root stringsDo: [:each |
		self assert: (each notEmpty and: [each allSatisfy: [:c | c isSeparator ]])].

	notPreserved := root nodes second.
	self assert: notPreserved nodes size = 1.

	preserved := notPreserved firstNode.
	self
		assert: preserved contentString notEmpty;
		assert: (preserved contentString allSatisfy: [:each | each isSeparator]).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/21/2014 05:58'!
testXMLIDAttribute
	| xml xmlWithDuplicateId |

	xml := '<a xml:id="one"><b xml:id="two"/></a>'.
	xmlWithDuplicateId := '<a xml:id="one"><b xml:id="one"/></a>'.
	self
		shouldnt: [SAXHandler parse: xml]
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: xmlWithDuplicateId]
		raise: XMLValidationException.
	self
		shouldnt: [
			(SAXHandler on: xmlWithDuplicateId)
				isValidating: false;
				parseDocument]
		raise: XMLValidationException.! !

TestCase subclass: #XMLTokenizerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLTokenizerTest methodsFor: 'sources' stamp: 'monty 1/12/2014 23:38'!
addressBookXML
	^ XMLParserTest addressBookXML! !

!XMLTokenizerTest methodsFor: 'sources' stamp: 'monty 1/12/2014 23:38'!
addressBookXMLWithDTD
	^ XMLParserTest addressBookXMLWithDTD! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 1/16/2014 09:08'!
testChangingNormalizedLineEnding
	| xml domParser parser tokenizer |

	xml :=
		'<root>{1}one{2}two{3}three{1}</root>'
			format: (Array with: String lf with: String cr with: String crlf).
	domParser := XMLDOMParser on: xml.

	parser := domParser parser.
	tokenizer := parser tokenizer.
	self assert: tokenizer normalizedLineEndingChar == Character lf.
	
	tokenizer normalizedLineEndingChar: $N.
	self assert: tokenizer normalizedLineEndingChar == $N.

	parser parseDocument.
	self assert: domParser document root contentString = 'NoneNtwoNthreeN'.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 2/10/2014 19:11'!
testConditionalSection
	| tokenizer |

	tokenizer :=
		XMLWellFormedParserTokenizer
			driver: nil
			on:
				'<!![INCLUDE [included',
				'<!![ IGNORE [ignored; <!![INCLUDE[ignored;]]> ignored.]]>',
				', included]]>, included.'.
	tokenizer tokenContext enterExternalSubset.
	tokenizer nextToken.
	self assert: tokenizer streamReader upToEnd = 'included, included, included.'.

	self
		should: [SAXHandler parse: '<!!DOCTYPE root [ <!![IGNORE[<!!--ignored-->]]> ]><root/>']
		raise: XMLWellFormednessException.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 2/10/2014 06:13'!
testTokenizerAddressBook
	| tokenizer |
	tokenizer :=
		XMLWellFormedParserTokenizer
			driver: nil
			on: self addressBookXML readStream.
	[tokenizer streamReader next notNil] whileTrue.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 2/10/2014 06:13'!
testTokenizerAddressBookWithDTD
	| tokenizer |
	tokenizer :=
		XMLWellFormedParserTokenizer
			driver: nil
			on: self addressBookXMLWithDTD readStream.
	[tokenizer streamReader next notNil] whileTrue.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 2/10/2014 02:43'!
testTokenizerCharacters
	| tokenizer |
	"This test is actually not that useful. This is not the proper way of using the parser.
	This test is here just for specification purpose"
	"self debug: #testParsingCharacters"

	tokenizer := XMLWellFormedParserTokenizer driver: nil on: self addressBookXML readStream.

	self
		assert: tokenizer streamReader next == $<;
		assert: ('address' allSatisfy: [:each | tokenizer streamReader next == each]);
		assert: tokenizer streamReader peek == $b.

	tokenizer streamReader pushBack: 'test'.
	self
		assert: ('test' allSatisfy: [:each | tokenizer streamReader next == each]);
		assert: tokenizer streamReader peek == $b;
		assert: ('book' allSatisfy: [:each | tokenizer streamReader next == each]).

	tokenizer streamReader
		pushBack: 'book';
		pushBack: 'test'.
	self
		assert: ('test' allSatisfy: [:each | tokenizer streamReader next == each]);
		assert: ('book' allSatisfy: [:each | tokenizer streamReader next == each]).! !

TestCase subclass: #XMLURITest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
absolute
	^ '/absolute/path'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
absoluteDir
	^ '/absolute/path/'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
absoluteDirRelativeCombined
	^ '/absolute/path/relative/path'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
absoluteRelativeCombined
	^ '/absolute/relative/path'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
domain
	^ 'http://www.example.com'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/21/2014 02:11'!
escaped
	^ '%20%22%3c%3E%25'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
fileHostURIString
	^ 'file://', self host, self absolute! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
fileURIString
	^ 'file://', self absolute! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
fragment
	^ '#fragmentID'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
fullURIString
	^ self domain, self absolute, self queryString, self fragment! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
host
	^ self domain copyAfterLast: $/! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
queryString
	^ '?name1=value1&name2=value2'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
relative
	^ 'relative/path'! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
scheme
	^ self domain copyUpTo: $:! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/8/2014 18:17'!
schemelessURIString
	^ (self domain copyAfter: $:), self absolute, self queryString, self fragment! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:08'!
testCombiningAbsoluteURIs
	| domainUri fullUri fileUri relativeUri |

	domainUri := self domain asXMLURI.
	fullUri := self fullURIString asXMLURI.
	fileUri := self fileURIString asXMLURI.
	relativeUri := self relative asXMLURI.
	self
		assert: (domainUri / self fullURIString) = fullUri;
		assert: (fullUri / self domain) = domainUri;
		assert: (fullUri / self fileURIString) = fileUri;
		assert: (relativeUri / self domain) = domainUri;
		assert: (relativeUri / self fullURIString) = fullUri;
		assert: (relativeUri / self fileURIString) = fileUri.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/16/2014 09:06'!
testCombiningDotPathURIs
	| absolutePathUri absoluteDirPathUri absoluteRelativeCombinedUri absoluteDirRelativeCombinedUri |

	absolutePathUri := self absolute asXMLURI.
	absoluteDirPathUri := self absoluteDir asXMLURI.
	absoluteRelativeCombinedUri := self absoluteRelativeCombined asXMLURI.
	absoluteDirRelativeCombinedUri := self absoluteDirRelativeCombined asXMLURI.
	self
		assert: (absolutePathUri / './path') = absolutePathUri;
		assert: (absolutePathUri / ('..', self absolute)) = absolutePathUri;
		assert: (absolutePathUri / ('./', self relative)) = absoluteRelativeCombinedUri;
		assert: (absolutePathUri / ('../absolute/', self relative)) = absoluteRelativeCombinedUri;
		assert: (absoluteDirPathUri / ('./', self relative)) = absoluteDirRelativeCombinedUri;
		assert: (absoluteDirPathUri / ('../path/', self relative)) = absoluteDirRelativeCombinedUri;
		assert: (absoluteDirRelativeCombinedUri / '../../path') = absolutePathUri;
		assert: (absoluteDirRelativeCombinedUri / ('../../..', self absolute)) = absolutePathUri.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/16/2014 08:55'!
testCombiningRelativeURIs
	| domainUri absolutePathUri absoluteDirPathUri |

	domainUri := self domain asXMLURI.
	absolutePathUri := (self domain, self absolute) asXMLURI.
	absoluteDirPathUri := (self domain, self absoluteDir) asXMLURI.
	self
		assert: (domainUri / self relative) = (self domain, '/', self relative) asXMLURI;
		assert: (domainUri / self absolute) = (self domain, self absolute) asXMLURI;
		assert: (domainUri / self absoluteDir) = (self domain, self absoluteDir) asXMLURI;
		assert: (absolutePathUri / self relative) = (self domain, self absoluteRelativeCombined) asXMLURI;
		assert: (absoluteDirPathUri / self relative) = (self domain, self absoluteDirRelativeCombined) asXMLURI;
		assert: (absolutePathUri / self queryString) = (self domain, self absolute, self queryString) asXMLURI;
		assert: (absolutePathUri / self fragment) = (self domain, self absolute, self fragment) asXMLURI;
		assert: (domainUri / self schemelessURIString) = self fullURIString asXMLURI.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:10'!
testConverting
	| fullUri |

	fullUri := XMLURI fromString: self fullURIString.

	self
		assert: self fullURIString asXMLURI = fullUri;
		assert: fullUri asXMLURI == fullUri.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:10'!
testParsingAbsolutePathURI
	| uri |

	uri := self absolute asXMLURI.
	self
		deny: uri isFileURI;
		assert: uri isRelative;
		assert: uri isLocalURI;
		assert: uri scheme isEmpty;
		assert: uri host isEmpty;
		assert: uri path = self absolute;
		assert: uri queryString isEmpty;
		assert: uri fragment isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/12/2014 23:05'!
testParsingDomainURI
	| uri |

	uri := self domain asXMLURI.
	self
		deny: uri isFileURI;
		deny: uri isRelative;
		deny: uri isLocalURI;
		assert: uri scheme = self scheme;
		assert: uri host = self host;
		assert: uri path isEmpty;
		assert: uri queryString isEmpty;
		assert: uri fragment isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:08'!
testParsingFileHostURI
	| uri |

	uri := self fileHostURIString asXMLURI.
	self
		deny: uri isRelative;
		assert: uri isFileURI;
		assert: uri isLocalURI;
		assert: uri scheme = 'file';
		assert: uri host = self host;
		assert: uri path = self absolute;
		assert: uri queryString isEmpty;
		assert: uri fragment isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:08'!
testParsingFileURI
	| uri |

	uri := self fileURIString asXMLURI.
	self
		deny: uri isRelative;
		assert: uri isFileURI;
		assert: uri isLocalURI;
		assert: uri scheme = 'file';
		assert: uri host isEmpty;
		assert: uri path = self absolute;
		assert: uri queryString isEmpty;
		assert: uri fragment isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/12/2014 23:06'!
testParsingFullURI
	| uri |

	uri := self fullURIString asXMLURI.
	self
		deny: uri isFileURI;
		deny: uri isRelative;
		deny: uri isLocalURI;
		assert: uri scheme = self scheme;
		assert: uri host = self host;
		assert: uri path = self absolute;
		assert: uri queryString = self queryString allButFirst;
		assert: uri fragment = self fragment allButFirst.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:08'!
testParsingRelativePathURI
	| uri |

	uri := self relative asXMLURI.
	self
		deny: uri isFileURI;
		assert: uri isRelative;
		assert: uri isLocalURI;
		assert: uri scheme isEmpty;
		assert: uri host isEmpty;
		assert: uri path = self relative;
		assert: uri queryString isEmpty;
		assert: uri fragment isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:07'!
testParsingURIWithoutScheme
	| uri |

	uri := self schemelessURIString asXMLURI.
	self
		deny: uri isFileURI;
		assert: uri isRelative;
		assert: uri isLocalURI;
		assert: uri scheme isEmpty;
		assert: uri host = self host;
		assert: uri path = self absolute;
		assert: uri queryString = self queryString allButFirst;
		assert: uri fragment = self fragment allButFirst.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/21/2014 02:11'!
testPathUnescaping
	| uri |

	uri := (self escaped, self relative, self escaped) asXMLURI.
	self
		assert: uri path = (self escaped, self relative, self escaped);
		assert: uri unescapedPath = (self unescaped, self relative, self unescaped);
		assert: '%' asXMLURI unescapedPath = '%'! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:04'!
testSchemeAndHostCaseSensitivity
	| uri uppercaseUri |

	uri := self fullURIString asXMLURI.
	uppercaseUri := self fullURIString asUppercase asXMLURI.
	self
		assert: uri scheme = uppercaseUri scheme;
		assert: uri host = uppercaseUri host;
		deny: uri path = uppercaseUri path;
		deny: uri queryString = uppercaseUri queryString;
		deny: uri fragment = uppercaseUri fragment.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/10/2014 06:07'!
testURIEquality
	| fullUri string |

	string := self fullURIString.
	fullUri := string asXMLURI.
	self
		assert: fullUri = fullUri;
		assert: fullUri = string asXMLURI;
		deny: fullUri = (string copyReplaceAll: self scheme with: 'new') asXMLURI;
		deny: fullUri = (string copyReplaceAll: self host with: 'new') asXMLURI;
		deny: fullUri = (string copyReplaceAll: self absolute with: 'new') asXMLURI;
		deny: fullUri = (string copyReplaceAll: self queryString with: 'new') asXMLURI;
		deny: fullUri = (string copyReplaceAll: self fragment with: 'new') asXMLURI.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2014 21:51'!
testURIPrinting
	self
		assert: self fullURIString asXMLURI printString = self fullURIString;
		assert: self schemelessURIString asXMLURI printString = self schemelessURIString;
		assert: self absolute asXMLURI printString = self absolute;
		assert: self relative asXMLURI printString = self relative;
		assert: self absoluteDir asXMLURI printString = self absoluteDir;
		assert: self fileURIString asXMLURI printString = self fileURIString;
		assert: self fileHostURIString asXMLURI printString = self fileHostURIString;
		assert: '/' asXMLURI printString = '/';
		assert: '' asXMLURI printString isEmpty.! !

!XMLURITest methodsFor: 'strings' stamp: 'monty 1/21/2014 02:10'!
unescaped
	^ ' "<>%'! !

XMLNodeWithElements subclass: #XMLMockNodeWithElements
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLMockNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:21'!
document
	^ nil! !

!XMLMockNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:36'!
document: aDocument
! !

XMLNode subclass: #XMLMockNode
	instanceVariableNames: 'notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLMockNode methodsFor: 'private' stamp: 'monty 2/2/2014 04:25'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLMockNode methodsFor: 'printing' stamp: 'JAA 3/24/2011 18:24'!
writeXMLOn: aWriter
	aWriter string: self class name asString, '(', self identityHash printString, ') '! !

XMLMockNode subclass: #XMLMockObserverNode
	instanceVariableNames: 'lastNotificationArguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLMockObserverNode methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 14:52'!
lastNotificationArguments
	^ lastNotificationArguments! !

!XMLMockObserverNode methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 14:52'!
lastNotificationArguments: anArray
	lastNotificationArguments := anArray! !

!XMLMockObserverNode methodsFor: 'notifications' stamp: 'JAAyer 10/28/2010 14:53'!
mockNotifyWith: aFirstObject
	lastNotificationArguments := Array with: aFirstObject! !

!XMLMockObserverNode methodsFor: 'notifications' stamp: 'JAAyer 10/28/2010 14:53'!
mockNotifyWith: aFirstObject with: aSecondObject
	lastNotificationArguments := Array with: aFirstObject with: aSecondObject! !

!XMLMockObserverNode methodsFor: 'notifications' stamp: 'JAAyer 10/28/2010 14:54'!
mockNotifyWith: aFirstObject with: aSecondObject with: aThirdArgument
	lastNotificationArguments := Array with: aFirstObject with: aSecondObject with: aThirdArgument! !

XMLNodeWithChildren subclass: #XMLMockNodeWithChildren
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLMockNodeWithChildren methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	^ XMLNodeList! !

Object subclass: #SAXHandlerProxy
	instanceVariableNames: 'saxHandler sentHandlerMessages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXHandlerProxy class methodsFor: 'instance creation' stamp: 'monty 12/6/2013 22:23'!
saxHandler: aSaxHandler
	^ self new saxHandler: aSaxHandler! !

!SAXHandlerProxy methodsFor: 'reflective operations' stamp: 'monty 12/7/2013 00:10'!
doesNotUnderstand: aMessage
	| result |

	(self handlerCategories anySatisfy: [:each |
		(saxHandler class selectorsInCategory: each)
			includes: aMessage selector])
		ifTrue: [self sentHandlerMessages addLast: aMessage].

	^ (result := aMessage sendTo: saxHandler) == saxHandler
		ifTrue: [self]
		ifFalse: [result].! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 12/6/2013 23:04'!
handlerCategories
	^ #(content lexical declaration)! !

!SAXHandlerProxy methodsFor: 'parsing' stamp: 'monty 12/6/2013 22:38'!
parseDocument
	self parser parseDocument! !

!SAXHandlerProxy methodsFor: 'parsing' stamp: 'monty 12/6/2013 22:37'!
parser
	^ XMLParser
		driver: (self driverClass saxHandler: self)
		on: saxHandler input! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 12/6/2013 22:23'!
saxHandler: aSaxHandler
	saxHandler := aSaxHandler! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 12/6/2013 23:50'!
sentHandlerMessages
	^ sentHandlerMessages ifNil: [sentHandlerMessages := OrderedCollection new]! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 12/6/2013 23:49'!
sentHandlerSelectors
	^ (self sentHandlerMessages collect: [:each | each selector]) asArray.! !

Object subclass: #XMLStubListObserver
	instanceVariableNames: 'notifications'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLStubListObserver methodsFor: 'notifications' stamp: 'JAA 3/26/2011 19:17'!
added: anObject to: aList
	self recordNotification! !

!XMLStubListObserver methodsFor: 'notifications' stamp: 'JAA 3/26/2011 19:23'!
adding: anObject to: aList
	self recordNotification! !

!XMLStubListObserver methodsFor: 'accessing' stamp: 'JAA 3/26/2011 19:16'!
notifications
	^ notifications ifNil: [notifications := OrderedCollection new]! !

!XMLStubListObserver methodsFor: 'notifications' stamp: 'JAA 3/26/2011 19:23'!
recordNotification
	self notifications add:
		(MessageSend
			receiver: self
			selector: thisContext sender selector
			arguments: thisContext sender arguments)! !

!XMLStubListObserver methodsFor: 'notifications' stamp: 'JAA 3/26/2011 19:23'!
removed: anObject from: aList
	self recordNotification! !
